#! /usr/bin/env python
# -*- coding: utf-8 -*-

import urllib2
import shutil
import os
import subprocess
import commands
import sys


def http_download(url, target_path):
    """Download file to local
    Args:
        - url(string): url request path
        - target_path(string): download destination

    Raises:
        IOError
        urllib2.URLError
    """
    try:
        resp = urllib2.urlopen(url)
    except urllib2.URLError, e:
        if not hasattr(e, 'code'):
            raise
        resp = e
    if resp.code != 200:
        raise IOError("Request url(%s) expect 200 but got %d" %(url, resp.code))

    with open(target_path, 'wb') as f:
        shutil.copyfileobj(resp, f)
    return target_path


# def run_cmd(*args, **kwargs):
#     """Run command and send stdout,stderr to console
#     Args:
#         ...command: ex   adb, devices
#         success_text(string): expect output must contains

#     Return:
#         exit code
#     """
#     output = subprocess.check_output(list(args))

#     success_text = kwargs.pop('success_text', None)
#     if success_text and output.find(success_text) == -1:
#         return 3
#     return 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import json
import time

from atx import consts

def record_operation(d, logdir='cloudtest', filename='operation.log'):
    """Record operation log to file for cloudtest
    Args:
        - d: atx device
        - logdir(string) : directory store pictures and log name
        - filename(string) : log file name

    Returns:
        function name
    """
    def listener(event):
        data = dict(
            action='screenshot',
            success=1,
            time=time.strftime('%Y-%m-%d %H:%M:%S'),
            content='截图',
            has_pic=False)
        log_path = os.path.join(logdir, filename)
        with open(log_path, 'a') as f:
            f.write(json.dumps(data)+'\n')

    if not os.path.isdir(logdir):
        os.makedirs(logdir)
    d.add_listener(listener, consts.EVENT_ALL)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
This is only for test airtest itself
'''

import os
import shutil
import airtest

from airtest import base
# from com.dtmilano.android.viewclient import ViewClient 
# from com.dtmilano.android.viewclient import adbclient

DEBUG = os.getenv("DEBUG")=="true"
log = base.getLogger('dummy')


class Device(object):
    def __init__(self, phoneno=None):
        self._snapshot = './default.png'
        self._text = ''
        self._click = None
        self._getCpu = False
        pass

    def snapshot(self, filename):
        ''' save screen snapshot '''
        log.debug('DUMMY take snapshot %s' %(filename))
        shutil.copyfile(self._snapshot, filename)

    def touch(self, x, y, duration=0.1):
        '''
        same as adb -s ${SERIALNO} shell input tap x y
        '''
        log.debug('touch position %s', (x, y))
        self._click = (x, y)

    def drag(self, (x0, y0), (x1, y1), duration=0.5):
        '''
        Drap screen
        '''
        log.debug('drap position %s -> %s', (x0, y0), (x1, y1))

    def shape(self):
        ''' 
        Get screen width and height 
        '''
        return 10, 20

    def type(self, text):
        '''
        Input some text

        @param text: string (text want to type)
        '''
        log.debug('type text: %s', repr(text))
        self._text = text

    def keyevent(self, event):
        '''
        Send keyevent by adb

        @param event: string (one of MENU, HOME, BACK)
        '''
        log.debug('keyevent: %s', event)

    def getMem(self, appname):
        return {}

    def getCpu(self, appname):
        self._getCpu = True
        return 0.0

    def start(self, dictSet):
        '''
        Start a program

        @param dictSet: dict (defined in air.json)
        '''
        log.debug('start %s', dictSet)

    def stop(self, dictSet):
        log.debug('stop %s', dictSet)

    def getdevinfo(self):
        return {
            'cpu_count': 1,
            'mem_total': 2,
            'mem_free': 111,
            'product_brand': 'dummy-brand',
            'product_model': 'dummy-model',
            }



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# License under MIT

from __future__ import absolute_import

import collections
import copy
import os
import re
import sys
import subprocess
import time
import tempfile
import warnings
import functools
import logging
import uuid
import inspect
import xml.dom.minidom

import cv2
import numpy as np
import aircv as ac
from uiautomator import AutomatorDeviceObject
from PIL import Image

from atx import consts
from atx import errors
from atx import patch
from atx import base
from atx import logutils
from atx import imutils
from atx import adb
from atx.device import Pattern, Bounds, FindPoint


__dir__ = os.path.dirname(os.path.abspath(__file__))
log = logutils.getLogger(__name__)

_Condition = collections.namedtuple('WatchCondition', ['pattern', 'exists'])

class WatcherItem(object):
    """
    How to use, for
    example:

    with d.watch('xx') as w:
        w.on('button.png').on('enter.png').click()
        w.on('yes.png').on_not('exit.png').click()

    or

    w = d.watch('yy')
    w.on('button.png').on('enter.png').click()
    w.on('button.png').click('enter.png')
    w.run()
    """
    def __init__(self, dev, done, handler, pattern):
        self._dev = dev
        self._done = done # list
        self._handler = handler
        self._hooks = handler['hooks'] = []
        self._conditions = handler['conditions'] = [_Condition(pattern, True)]

    def on(self, pattern):
        """Trigger when pattern exists"""
        self._conditions.append(_Condition(pattern, True))
        return self

    def on_not(self, pattern):
        """Trigger when pattern not exists"""
        self._conditions.append(_Condition(pattern, False))
        return self

    def do(self, func):
        """Trigger with function call
        Args:
            func: function which will called when object found. For example.

            def foo(event):
                print event.pos # (x, y) position
            
            w.on('kitty.png').do(foo)
        
        Returns:
            Watcher object

        Raises:
            SyntaxError
        """
        if not callable(func):
            raise SyntaxError("%s should be a function" % func)
        self._hooks.append(func)
        return self

    def click(self, *args, **kwargs):
        def _inner(event):
            if len(args) or len(kwargs):
                self._dev.click(*args, **kwargs)
            else:
                self._dev.click(*event.pos)
        return self.do(_inner)

    def click_image(self, *args, **kwargs):
        """ async trigger click_image """
        def _inner(event):
            return self._dev.click_image(*args, **kwargs)

        return self.do(_inner)

    def quit(self):
        def _inner(event):
            self._done[0] = True
            # raise RuntimeError("Not finished yet.")
        return self.do(_inner)

class Watcher(object):
    ACTION_CLICK = 1 <<0
    ACTION_TOUCH = 1 <<0
    ACTION_QUIT = 1 <<1

    Handler = collections.namedtuple('Handler', ['selector', 'action'])
    Event = collections.namedtuple('Event', ['selector', 'pos'])

    def __init__(self, device, name=None, timeout=None):
        self._events = []
        self._dev = device
        self._run = False
        self._stored_selector = None

        self._wids = [] # store orders
        self._handlers = {}
        self._done = [False]

        self.name = name
        self.touched = {}
        self.timeout = timeout

    def on(self, pattern):
        # TODO(ssx): maybe an array is just enough
        watch_id = str(uuid.uuid1())
        self._wids.append(watch_id)
        handler = self._handlers[watch_id] = {}
        return WatcherItem(self._dev, self._done, handler, pattern)

    def _do_hook(self, screen):
        patterns = set()
        for handler in self._handlers.values():
            for c in handler['conditions']:
                patterns.add(c.pattern)

        # TODO(ssx): here can have a better optimized way.
        # no need to match all pattern all the time
        matches = {}
        for pattern in patterns:
            matches[pattern] = self._match(pattern, screen)

        for wid in self._wids:
            hdlr = self._handlers[wid]
            # pos = self._match(handler.pattern)
            # if pos is None:
            #     continue

            last_pos = None
            ok = True
            for c in hdlr['conditions']:
                last_pos = matches[c.pattern]
                exists = last_pos is not None
                if exists != c.exists:
                    ok = False
                    break
            if ok:
                for fn in hdlr['hooks']:
                    fn(Watcher.Event(None, last_pos))

    def run(self):
        # self._run = True
        start_time = time.time()
        while not self._done[0]:
            screen = self._dev.screenshot()
            self._do_hook(screen)

            if self.timeout is not None:
                if time.time() - start_time > self.timeout:
                    raise errors.WatchTimeoutError("Watcher(%s) timeout %s" % (self.name, self.timeout,))
                sys.stdout.write("Watching %4.1fs left: %4.1fs\r" %(self.timeout, self.timeout-time.time()+start_time))
                sys.stdout.flush()
            sys.stdout.write('\n')

    # def on_old(self, pattern=None, text=None):
    #     """Trigger when some object exists
    #     Args:
    #         image: image filename or Pattern
    #         text: For uiautomator

    #     Returns:
    #         None

    #     Raises:
    #         TypeError
    #     """
    #     if text:
    #         self._stored_selector = self._dev(text=text)
    #     elif pattern is not None:
    #         selector = self._dev.pattern_open(pattern)
    #         if selector is None:
    #             raise IOError("Not found pattern: {}".format(pattern))
    #         self._stored_selector = selector
    #     else:
    #         raise TypeError("unsupported type: %s", pattern)
            
    #     return self

    def touch(self):
        return self.click()

    def click(self):
        """Touch"""
        self._events.append(Watcher.Handler(self._stored_selector, Watcher.ACTION_CLICK))
        return self

    def quit(self):
        self._events.append(Watcher.Handler(self._stored_selector, Watcher.ACTION_QUIT))

    def do(self, func):
        """Trigger with function call
        Args:
            func: function which will called when object found. For example.

            def foo(event):
                print event.pos # (x, y) position
            
            w.on('kitty.png').do(foo)
        
        Returns:
            Watcher object

        Raises:
            SyntaxError
        """
        if not callable(func):
            raise SyntaxError("%s should be a function" % func)
        self._events.append(Watcher.Handler(self._stored_selector, func))
        return self

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        # print self._handlers
        # self._run_watch()
        self.run()

    def _match(self, selector, screen):
        ''' Find position for AtxPattern or UIAutomator Object

        Return:
            position(x, y) or None
        '''
        if isinstance(selector, Pattern) or isinstance(selector, basestring):
            ret = self._dev.match(selector, screen=screen)
            log.debug('watch match: %s, confidence: %s', selector, ret)
            if ret is None:
                return None
            return ret.pos
        elif isinstance(selector, AutomatorDeviceObject):
            if not selector.exists:
                return None
            info = selector.info['bounds']
            x = (info['left'] + info['right']) / 2
            y = (info['bottom'] + info['top']) / 2
            return (x, y)

    # def _hook(self, screen):
    #     for evt in self._events:
    #         pos = self._match(evt.selector, screen)
    #         if pos is None:
    #             continue

    #         if callable(evt.action):
    #             evt.action(Watcher.Event(evt.selector, pos))
    #         elif evt.action == Watcher.ACTION_CLICK:
    #             log.info('Watch match %s, click: %s', evt.selector, pos)
    #             self._dev.click(*pos)
    #         elif evt.action == Watcher.ACTION_QUIT:
    #             self._run = False

    # def _run_watch(self):
    #     self._run = True
    #     start_time = time.time()
        
    #     while self._run:
    #         screen = self._dev.screenshot()
    #         self._hook(screen)
    #         if self.timeout is not None:
    #             if time.time() - start_time > self.timeout:
    #                 raise errors.WatchTimeoutError("Watcher(%s) timeout %s" % (self.name, self.timeout,))
    #             sys.stdout.write("Watching %4.1fs left: %4.1fs\r" %(self.timeout, self.timeout-time.time()+start_time))
    #             sys.stdout.flush()
    #     sys.stdout.write('\n')


HookEvent = collections.namedtuple('HookEvent', ['flag', 'args', 'kwargs'])

def hook_wrap(event_type):
    def wrap(fn):
        @functools.wraps(fn)
        def _inner(*args, **kwargs):
            func_args = inspect.getcallargs(fn, *args, **kwargs)
            self = func_args.get('self')
            if self and hasattr(self, '_listeners'):
                for (f, event_flag) in self._listeners:
                    if event_flag & event_type:
                        f(HookEvent(event_flag, args[1:], kwargs)) # remove self from args
            return fn(*args, **kwargs)
        return _inner
    return wrap

class DeviceMixin(object):
    def __init__(self):
        self.image_match_method = consts.IMAGE_MATCH_METHOD_TMPL
        self.image_match_threshold = 0.8
        self._resolution = None
        self._bounds = None
        self._listeners = []
        self.image_path = ['.']

    @property
    def resolution(self):
        return self._resolution

    @resolution.setter
    def resolution(self, value):
        if value is None:
            self._resolution = None
        else:
            if not isinstance(value, tuple) or len(value) != 2:
                raise TypeError("Value should be tuple, contains two values")
            self._resolution = tuple(sorted(value))
    
    def pattern_open(self, image):
        if isinstance(image, Pattern):
            return image
        elif isinstance(image, basestring):
            image_path = base.search_image(image, self.image_path)
            if image_path is None:
                raise IOError('image file not found: {}'.format(image))
            return Pattern(image_path)
        elif 'numpy' in str(type(image)):
            return Pattern(image)
        else:
            raise TypeError("Not supported image type: {}".format(type(image)))

    def delay(self, secs):
        """Delay some seconds
        Args:
            secs: float seconds

        Returns:
            self
        """
        secs = int(secs)
        for i in reversed(range(secs)):
            sys.stdout.write('\r')
            sys.stdout.write("sleep %ds, left %2ds" % (secs, i+1))
            sys.stdout.flush()
            time.sleep(1)
        sys.stdout.write("\n")
        return self

    def exists(self, pattern, screen=None):
        """Check if image exists in screen

        Returns:
            If exists, return FindPoint, or
            return None if result.confidence < self.image_match_threshold
        """
        ret = self.match(pattern, screen)
        if ret is None:
            return None
        if not ret.matched:
            return None
        return ret

    def wait(self, pattern, timeout=10.0):
        """Wait till pattern is found or time is out (default: 10s)."""
        t = time.time() + timeout
        while time.time() < t:
            ret = self.exists(pattern)
            if ret:
                return ret
            time.sleep(0.2)
        raise errors.ImageNotFoundError('Not found image %s' %(pattern,))

    def touch(self, x, y):
        """ Alias for click """
        self.click(x, y)

    def _cal_scale(self, pattern=None):
        scale = 1.0
        resolution = (pattern and pattern.resolution) or self.resolution
        if resolution is not None:
            ow, oh = sorted(resolution)
            dw, dh = sorted(self.display)
            fw, fh = 1.0*dw/ow, 1.0*dh/oh
            # For horizontal screen, scale by Y (width)
            # For vertical screen, scale by X (height)
            scale = fw if self.rotation in (1, 3) else fh
        return scale

    @property
    def bounds(self):
        if self._bounds is None:
            return None
        return self._bounds * self._cal_scale()
    
    def match(self, pattern, screen=None, threshold=None):
        """Check if image position in screen

        Args:
            - pattern: Image file name or opencv image object
            - screen: opencv image, optional, if not None, screenshot method will be called

        Returns:
            None or FindPoint, For example:

            FindPoint(pos=(20, 30), method='tmpl', confidence=0.801, matched=True)

            Only when confidence > self.image_match_threshold, matched will be True

        Raises:
            TypeError: when image_match_method is invalid
        """
        pattern = self.pattern_open(pattern)
        search_img = pattern.image

        pattern_scale = self._cal_scale(pattern)
        if pattern_scale != 1.0:
            search_img = cv2.resize(search_img, (0, 0), 
                fx=pattern_scale, fy=pattern_scale,
                interpolation=cv2.INTER_CUBIC)
        
        screen = screen or self.region_screenshot()
        threshold = threshold or self.image_match_threshold

        dx, dy = pattern.offset
        dx, dy = int(dx*pattern_scale), int(dy*pattern_scale)

        # image match
        screen = imutils.from_pillow(screen) # convert to opencv image
        match_method = self.image_match_method
        ret = None
        if match_method == consts.IMAGE_MATCH_METHOD_TMPL:
            ret = ac.find_template(screen, search_img)
        elif match_method == consts.IMAGE_MATCH_METHOD_SIFT:
            ret = ac.find_sift(screen, search_img, min_match_count=10)
        else:
            raise TypeError("Invalid image match method: %s" %(match_method,))

        if ret is None:
            return None
        (x, y) = ret['result']
        # fix by offset
        position = (x+dx, y+dy)
        if self.bounds:
            x, y = position
            position = (x+self.bounds.left, y+self.bounds.top)
        confidence = ret['confidence']

        matched = True
        if match_method == consts.IMAGE_MATCH_METHOD_TMPL:
            if confidence < threshold:
                matched = False
        elif match_method == consts.IMAGE_MATCH_METHOD_SIFT:
            matches, total = confidence
            if 1.0*matches/total > 0.5: # FIXME(ssx): sift just write here
                matched = True
        return FindPoint(position, confidence, match_method, matched=matched)

    def region(self, bounds):
        """Set region of the screen area
        Args:
            bounds: Bounds object

        Returns:
            A new AndroidDevice object

        Raises:
            TypeError
        """
        if not isinstance(bounds, Bounds):
            raise TypeError("region param bounds must be isinstance of Bounds")
        _d = copy.copy(self)
        _d._bounds = bounds
        return _d

    def region_screenshot(self, filename=None):
        if self._bounds is None:
            return self.screenshot(filename)
        screen = self.screenshot()
        screen_crop = screen.crop(self.bounds)
        if filename:
            screen_crop.save(filename)
        return screen_crop

    def touch_image(self, *args, **kwargs):
        """ALias for click_image"""
        self.click_image(*args, **kwargs)

    def add_listener(self, fn, event_flags):
        """Listen event
        Args:
            - fn: function call when event happends
            - event_flags: for example
                EVENT_UIAUTO_CLICK | EVENT_UIAUTO_SWIPE

        Returns:
            None
        """
        self._listeners.append((fn, event_flags))

    def _trigger_event(self, event_flag, event):
        for (fn, flag) in self._listeners:
            if flag & event_flag:
                fn(event)

    def assert_exists(self, pattern, timeout=20.0):
        """Assert if image exists
        Args:
            - image: image filename # not support pattern for now
            - timeout (float): seconds

        Returns:
            self

        Raises:
            AssertExistsError
        """
        pattern = self.pattern_open(pattern)
        search_img = pattern.image
        # search_img = imutils.open(image)
        log.info('assert exists image: %s', pattern)
        start_time = time.time()
        while time.time() - start_time < timeout:
            point = self.match(search_img)
            if point is None:
                sys.stdout.write('.')
                sys.stdout.flush()
                continue
            if not point.matched:
                log.debug('Ignore confidence: %s', point.confidence)
                continue
            log.debug('assert pass, confidence: %s', point.confidence)
            sys.stdout.write('\n')
            break
        else:
            sys.stdout.write('\n')
            raise errors.AssertExistsError('image not found %s' %(pattern,))

    @hook_wrap(consts.EVENT_CLICK_IMAGE)
    def click_image(self, pattern, timeout=20.0, wait_change=False):
        """Simulate click according image position

        Args:
            - pattern (str or Pattern): filename or an opencv image object.
            - timeout (float): if image not found during this time, ImageNotFoundError will raise.
            - wait_change (bool): wait until background image changed.

        Returns:
            None

        Raises:
            ImageNotFoundError: An error occured when img not found in current screen.
        """
        pattern = self.pattern_open(pattern)
        log.info('click image: %s', pattern)
        start_time = time.time()
        found = False
        while time.time() - start_time < timeout:
            point = self.match(pattern)
            if point is None:
                sys.stdout.write('.')
                sys.stdout.flush()
                continue
            if not point.matched:
                log.info('Ignore confidence: %s', point.confidence)
                continue
            log.debug('confidence: %s', point.confidence)
            self.touch(*point.pos)
            self._trigger_event(consts.EVENT_UIAUTO_CLICK, point)
            found = True
            break
        sys.stdout.write('\n')

        # wait until click area not same
        if found and wait_change:
            start_time = time.time()
            while time.time()-start_time < timeout:
                # screen_img = self.screenshot()
                ret = self.match(pattern)
                if ret is None:
                    break
        if not found:
            raise errors.ImageNotFoundError('Not found image %s' %(pattern,))

    def watch(self, name, timeout=None):
        """Return a new watcher
        Args:
            name: string watcher name
            timeout: watch timeout

        Returns:
            watcher object
        """
        w = Watcher(self, name, timeout)
        w._dev = self
        return w

if __name__ == '__main__':
    b = Bounds(1, 2, 3, 4)
    print b
    print b * 1.0


# coding: utf-8
#

import atx
from PIL import Image

d = atx.connect('127.0.0.1:26944')
d.resolution = (720, 1280)
d.screenshot('screen.png')
bg = None #Image.open('hm.png')
print d.match('folder.png', bg)
nd = d.region(atx.Bounds(12, 324, 646, 418))
print d._bounds
print nd.match('folder.png', bg)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# USAGE
# python -matx -s ESLKJXX gui

import argparse
import functools
import json

import atx.androaxml as apkparse

from atx.cmds import run
from atx.cmds import iosdeveloper
from atx.cmds import install


def parser(name=None, debug=False):
    return wrap


def make_parser(debug=False):
    m = {}

    def wrap(fn):
        @functools.wraps(fn)
        def _inner(*args, **kwargs):
            try:
                return fn(*args, **kwargs)
            except Exception as e:
                if debug:
                    raise
                print 'ERROR: %s' % e
                raise SystemExit(1)

        m[fn.__name__] = _inner
        return _inner
    return (m, wrap)


funcs, wrap = make_parser(debug=True)


# serial, host, port, other args
# args: {serial: 'EFSF', host: '127.0.0.1', port: 5037, path: 'demo.apk'}
# name: gui
def parser_gui(serial, host, port, path):
    pass


def _gui(args):
    from atx.cmds import tkgui
    tkgui.main(args.serial, host=args.host)


def _minicap(args):
    from atx.cmds import minicap
    minicap.install(args.serial, host=args.host, port=args.port)


def _tcpproxy(args):
    from atx.cmds import tcpproxy
    tcpproxy.main(local_port=args.forward, listen_port=args.listen)


def _webide(args):
    from atx.cmds import webide
    webide.main(open_browser=(not args.no_browser), port=args.web_port, adb_host=args.host, adb_port=args.port)


def _monkey(args):
    from atx.cmds import monkey
    monkey.main(args.serial, args.host, args.port)


def _apk_parse(args):
    (pkg_name, activity) = apkparse.parse_apk(args.filename)
    print json.dumps({
        'package_name': pkg_name,
        'main_activity': activity,
    }, indent=4)


def _apk_install(args):
    install.main(args.path, serial=args.serial, host=args.host, port=args.port, start=args.start)


def _iosdeveloper(args):
    iosdeveloper.main(args)

def _record(args):
    # because record contains win32api which not working on unix system
    from atx.cmds import record
    record.main()

def _run(args):
    run.main(args.config_file)


def main():
    ap = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    ap.add_argument("-s", "--serial", "--udid", required=False, help="Android serial or iOS unid")
    ap.add_argument("-H", "--host", required=False, default='127.0.0.1', help="Adb host")
    ap.add_argument("-P", "--port", required=False, type=int, default=5037, help="Adb port")

    subparsers = ap.add_subparsers()
    add_parser = functools.partial(subparsers.add_parser, 
            formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_gui = add_parser('gui')
    parser_gui.set_defaults(func=_gui)

    parser_minicap = add_parser('minicap')
    parser_minicap.set_defaults(func=_minicap)

    parser_tcpproxy = add_parser('tcpproxy')
    parser_tcpproxy.add_argument('-l', '--listen', default=5555, type=int, help='Listen port')
    parser_tcpproxy.add_argument('-f', '--forward', default=26944, type=int, help='Forwarded port')
    parser_tcpproxy.set_defaults(func=_tcpproxy)

    parser_web = add_parser('web')
    parser_web.add_argument('--no-browser', dest='no_browser', action='store_true', help='Not open browser')
    parser_web.add_argument('--port', dest='web_port', default=None, type=int, help='web listen port')
    parser_web.set_defaults(func=_webide)
    
    parser_apk = add_parser('apkparse')
    parser_apk.add_argument('filename', help='Apk filename')
    parser_apk.set_defaults(func=_apk_parse)

    parser_ins = add_parser('install')
    parser_ins.add_argument('path', help='<apk file path | apk url path> (only support android for now)')
    parser_ins.add_argument('--start', action='store_true', help='Start app when app success installed')
    parser_ins.set_defaults(func=_apk_install)

    parser_run = add_parser('run')
    parser_run.add_argument('-f', dest='config_file', default='atx.yml', help='config file')
    parser_run.set_defaults(func=_run)

    parser_ios = add_parser('iosdeveloper')
    parser_ios.add_argument('-u', '--udid', required=False, help='iOS udid')
    parser_ios.set_defaults(func=_iosdeveloper)

    parse_record = add_parser('record')
    parse_record.set_defaults(func=_record)

    parse_monkey = add_parser('monkey')
    parse_monkey.set_defaults(func=_monkey)

    args = ap.parse_args()
    args.func(args)


if __name__ == '__main__':
    main()



# USAGE
# python click_and_crop.py --image jurassic_park_kitchen.jpg

# import the necessary packages
import argparse
import cv2
import Tkinter
import tkSimpleDialog
from PIL import ImageTk, Image
from StringIO import StringIO


def make_mouse_callback(imgs, ref_pt):
    # initialize the list of reference points and boolean indicating
    # whether cropping is being performed or not
    cropping = [False]
    clone = imgs[0]

    def _click_and_crop(event, x, y, flags, param):
        # grab references to the global variables
        # global ref_pt, cropping

        # if the left mouse button was clicked, record the starting
        # (x, y) coordinates and indicate that cropping is being
        # performed
        if event == cv2.EVENT_LBUTTONDOWN:
            ref_pt[0] = (x, y)
            cropping[0] = True

        # check to see if the left mouse button was released
        elif event == cv2.EVENT_LBUTTONUP:
            # record the ending (x, y) coordinates and indicate that
            # the cropping operation is finished
            ref_pt[1] = (x, y)
            cropping[0] = False

            # draw a rectangle around the region of interest
            imgs[1] = image = clone.copy()
            cv2.rectangle(image, ref_pt[0], ref_pt[1], (0, 255, 0), 2)
            cv2.imshow("image", image)
        elif event == cv2.EVENT_MOUSEMOVE and cropping[0]:
            img2 = clone.copy()
            cv2.rectangle(img2, ref_pt[0], (x, y), (0, 255, 0), 2)
            imgs[1] = image = img2
            cv2.imshow("image", image)
    return _click_and_crop

def interactive_save(image):
    img_str = cv2.imencode('.png', image)[1].tostring()
    imgpil = Image.open(StringIO(img_str))

    root = Tkinter.Tk()
    root.geometry('{}x{}'.format(400, 400))
    imgtk = ImageTk.PhotoImage(image=imgpil)
    panel = Tkinter.Label(root, image=imgtk) #.pack()
    panel.pack(side="bottom", fill="both", expand="yes")
    Tkinter.Button(root, text="Hello!").pack()
    save_to = tkSimpleDialog.askstring("Save cropped image", "Enter filename")
    if save_to:
        if save_to.find('.') == -1:
            save_to += '.png'
        print 'Save to:', save_to
        cv2.imwrite(save_to, image)
    root.destroy()

def main():
    # construct the argument parser and parse the arguments
    ap = argparse.ArgumentParser()
    ap.add_argument("-i", "--image", required=True, help="Path to the image")
    args = vars(ap.parse_args())

    # load the image, clone it, and setup the mouse callback function
    image = cv2.imread(args["image"])
    clone = image.copy()
    images = [clone, image]
    ref_pt = [None, None]
    
    cv2.namedWindow("image")
    cv2.setMouseCallback("image", make_mouse_callback(images, ref_pt))

    # keep looping until the 'q' key is pressed
    while True:
        # display the image and wait for a keypress
        cv2.imshow("image", images[1])
        key = cv2.waitKey(1) & 0xFF

        # if the 'c' key is pressed, break from the loop
        if key == ord("c"):
            if ref_pt[1] is None:
                continue
            roi = clone[ref_pt[0][1]:ref_pt[1][1], ref_pt[0][0]:ref_pt[1][0]]
            interactive_save(roi)
        elif key == ord("q"):
            break

    # if there are two reference points, then crop the region of interest
    # from teh image and display it

    # close all open windows
    cv2.destroyAllWindows()

if __name__ == '__main__':
    main()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
TODO: not finished
basic operation for a game(like a user does)
'''

# import os
import re
import urllib
import requests

# from .. import base
# log = base.getLogger('androidwifi')

class Device(object):
    def __init__(self, addr=None):
        self._addr = addr
        if not self._addr.startswith('http://'):
            self._addr = 'http://'+self._addr

    def _req(self, uri, params):
        ''' Make a GET Request '''
        ret = requests.get(self._addr+'/'+uri, params=params)
        print 'DEUBG(%s, %s): %s' %(uri, params, ret.text)
        return ret

    def snapshot(self, filename):
        ''' save screen snapshot '''
        urllib.urlretrieve(self._addr+'/screen.png', filename)

    def touch(self, x, y, duration=0.1):
        '''
        touch position
        '''
        # self._req('click', dict(x=x, y=y, event=eventType))
        pass

    def drag(self, (x0, y0), (x1, y1), duration=0.5):
        '''
        Drap screen
        '''
        self._req('drag', dict(x0=x0,y0=y0,x1=x1,y1=y1,duration=duration))
        

    def shape(self):
        ''' 
        Get screen width and height 
        '''
        return (0, 0)

    def type(self, text):
        '''
        Input some text

        @param text: string (text want to type)
        '''
        # log.debug('type text: %s', repr(text))
        pass

    def keyevent(self, event):
        '''
        Send keyevent by adb

        @param event: string (one of MENU, HOME, BACK)
        '''
        self.adb.shell('input keyevent '+str(event))

    def meminfo(self, appname):
        '''
        Retrive memory info for app
        @param package(string): android package name
        @return dict: {'VSS', 'RSS', 'PSS'} (unit KB)
        '''
        return dict(VSS=1)

    def cpuinfo(self, appname):
        '''
        @param package(string): android package name
        @return dict: {'total': float, 'average': float}
        '''
        total=100
        return dict(total=total, average=total/self._devinfo['cpu_count'])

    def getdevinfo(self):
        # cpu
        output = self.adb.shell('cat /proc/cpuinfo')
        matches = re.compile('processor').findall(output)
        cpu_count = len(matches)
        # mem
        output = self.adb.shell('cat /proc/meminfo')
        match = re.compile('MemTotal:\s*(\d+)\s*kB\s*MemFree:\s*(\d+)', re.IGNORECASE).match(output)
        if match:
            mem_total = int(match.group(1), 10)>>10 # MB
            mem_free = int(match.group(2), 10)>>10
        else:
            mem_total = -1
            mem_free = -1

        # brand = self.adb.getProperty('ro.product.brand')
        return {
            'cpu_count': cpu_count,
            'mem_total': mem_total,
            'mem_free': mem_free,
            'product_brand': self.adb.getProperty('ro.product.brand'),
            'product_model': self.adb.getProperty('ro.product.model')
            }

if __name__ == '__main__':
    dev = Device('http://10.242.115.60:8765')
    dev.touch(1, 2)
    dev.drag((1,2), (3,4))


#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
basic operation for a game(like a user does)
'''

import json
import os
import re
import subprocess
import string
import StringIO
from functools import partial
from .adb.adbclient import AdbClient

from .. import patch, base
from .. import proto

DEBUG = os.getenv("AIRDEBUG")=="true"
log = base.getLogger('android')

__dir__ = os.path.dirname(os.path.abspath(__file__))

def str2any(s):
    if s.isdigit():
        return int(s)
    if s.lower() == 'true' or s.lower() == 'false':
        return s.lower() == 'true'
    return s

class Monitor(object):
    def __init__(self, serialno, pkgname):
        self._sno = serialno 
        self._pkg = pkgname
        def _adb(*args):
            return subprocess.check_output(['adb', '-s', self._sno] + list(args))
        self.adb = _adb
        self.adbshell = partial(_adb, 'shell')

    @patch.run_once
    def ncpu(self):
        ''' number of cpu '''
        output = self.adbshell('cat', '/proc/cpuinfo')
        matches = re.compile('processor').findall(output)
        return len(matches)

    def cpu(self, percpu=False):
        ''' cpu usage, range must be in [0, 100] '''      
        for line in StringIO.StringIO(self.adbshell('dumpsys', 'cpuinfo')):
            line = line.strip()
            # 0% 11655/im.yixin: 0% user + 0% kernel / faults: 10 minor
            if '/'+self._pkg+':' in line:
                return float(line.split()[0][:-1])/self.ncpu()
        return None

    def sys_cpu(self, percpu=False):
        ''' use air-native '''
        if percpu:
            output = self.adbshell(proto.AIRNATIVE, '-q', '-runjs', 
                'console.log(JSON.stringify(cpuPercent(300, true)))')
            return json.loads(output)
        else:
            output = self.adbshell(proto.AIRNATIVE, '-q', '-runjs', 
                'console.log(JSON.stringify(cpuPercent(300, false)))')
            return json.loads(output)[0]

    def battery(self):
        ''' use: adb shell dumpsys battery '''
        output = self.adbshell('dumpsys', 'battery')
        output = output[output.find('\n'):] # ignore the first line
        patten = re.compile('(\w[\w ]+):\s*([-\w\d]+)')
        ret = {}
        for key, val in patten.findall(output):
            ret[key] = str2any(val)
        return ret

    def memory(self):
        '''
        @description details view: http://my.oschina.net/goskyblue/blog/296798

        @param package(string): android package name
        @return dict: {'VSS', 'RSS', 'PSS'} (unit KB)
        '''
        ret = {}
        # VSS, RSS
        for line in StringIO.StringIO(self.adbshell('ps')):
            if line and line.split()[-1] == self._pkg:
                # USER PID PPID VSIZE RSS WCHAN PC NAME
                values = line.split()
                if values[3].isdigit() and values[4].isdigit():
                    ret.update(dict(VSS=int(values[3]), RSS=int(values[4])))
                else:
                    ret.update(dict(VSS=-1, RSS=-1))
                break
        else:
            log.error("mem get: adb shell ps error")
            return {}

        # PSS
        memout = self.adbshell('dumpsys', 'meminfo', self._pkg)
        pss = 0
        result = re.search(r'\(Pss\):(\s+\d+)+', memout, re.M)
        if result:
            pss = result.group(1)
        else:
            result = re.search(r'TOTAL\s+(\d+)', memout, re.M)
            if result:
                pss = result.group(1)
        ret.update(dict(PSS=int(pss)))
        return ret

    def sys_memory(self):
        '''
        unit KB
        '''
        output = self.adbshell('cat', '/proc/meminfo')
        match = re.compile('MemTotal:\s*(\d+)\s*kB\s*MemFree:\s*(\d+)', re.IGNORECASE).match(output)
        if match:
            total = int(match.group(1), 10)
            free = int(match.group(2), 10)
        else:
            total, free = 0, 0
        return dict(TOTAL=total, FREE=free)

class Device(object):
    def __init__(self, serialno, addr=''):
        self._snapshot_method = 'adb'
        self._serialno = serialno
        print 'SerialNo:', serialno

        if addr:
            host, port = addr.split(':')
            self.adbclient = AdbClient(serialno, hostname=host, port=int(port))
        else:
            self.adbclient = AdbClient(serialno)

        # self.adbclient, self._serialno = ViewClient.connectToDeviceOrExit(verbose=False, serialno=serialno, ignoreversioncheck=True)
        self.adbclient.setReconnect(True) # this way is more stable

        # self.vc = ViewClient(self.adbclient, serialno, autodump=False)

        def _adb(*args):
            if addr:
                return subprocess.check_output(['adb', '-H', host, '-P', port, '-s', self._serialno] + list(map(str, args)))
            return subprocess.check_output(['adb', '-s', self._serialno] + list(map(str, args)))
        self.adb = _adb
        self.adbshell = partial(_adb, 'shell')
      
        # try:
        #     if not self.adbclient.isScreenOn():
        #         self.adb.wake()
        # except:
        #     pass

        # install air-native
        airnative = os.path.join(__dir__, '../binfiles/air-native')
        md5sum = open(airnative+'.md5').read().strip()
        try:
            output = self.adbshell('md5', proto.AIRNATIVE)
            arr = string.split(output, maxsplit=1)
        except:
            arr = [None]
        if arr and md5sum != arr[0]:
            self.adb('push', airnative, proto.AIRNATIVE)
            self.adbshell('chmod', '755', proto.AIRNATIVE)

        self._init_adbinput()

    def _init_adbinput(self):
        apkfile = os.path.join(__dir__, '../binfiles/adb-keyboard.apk')
        pkgname = 'com.android.adbkeyboard'
        if not self.adbshell('pm', 'path', pkgname).strip():
            print 'Install adbkeyboard.apk input method'
            self.adb('install', '-r', apkfile)

    def snapshot(self, filename):
        ''' save screen snapshot '''
        if self._snapshot_method == 'adb':
            log.debug('start take snapshot(%s)'%(filename))
            self.adbclient.display['orientation'] = self.rotation()
            pil = self.adbclient.takeSnapshot(reconnect=True)
            pil.save(filename)
        elif self._snapshot_method == 'screencap':
            # FIXME(ssx): image not rotate
            tmpname = '/data/local/tmp/airtest-tmp-snapshot.png'
            self.adbshell('screencap', '-p', tmpname)
            self.adb('pull', tmpname, filename)
            self.adbshell('rm', tmpname)
        else:
            raise RuntimeError("No such snapshot method: [%s]" % self._snapshot_method)


    def touch(self, x, y, duration=None):
        '''
        same as adb -s ${SERIALNO} shell input tap x y
        '''
        if not duration:
            self.adbshell('input', 'tap', x, y)
        else:
            self.adbshell(proto.AIRNATIVE, '-runjs', 'tap({x}, {y}, {dur})'.format(
                x=x, y=y, dur=int(duration*1000)))

    def drag(self, (x0, y0), (x1, y1), duration=0.5):
        '''
        Drap screen
        '''
        self.adbshell(proto.AIRNATIVE, '-runjs', 'drag({x0}, {y0}, {x1}, {y1}, {steps}, {dur})'.format(
            x0=x0, y0=y0, x1=x1, y1=y1, steps=10, dur=int(duration*1000)))
        # self.adb.drag((x0, y0), (x1, y1), duration)

    def rotation(self):
        '''
        dumpsys ref: http://imsardine.simplbug.com/note/android/adb/commands/dumpsys.html
        '''
        patten = re.compile('SurfaceOrientation:\s+(\d+)')
        output = self.adbshell('dumpsys', 'input')
        match = patten.search(output)
        if match:
            return int(match.group(1))
        return proto.ROTATION_0

    def shape(self):
        ''' 
        Get screen width and height, when landscape width > height
        '''
        width = self.adbclient.getProperty("display.width")
        height = self.adbclient.getProperty("display.height")
        return (width, height)

    def type(self, text):
        '''
        Input some text

        @param text: string (text want to type)
        '''
        log.debug('type text: %s', repr(text))
        adbime = 'com.android.adbkeyboard/.AdbIME'
        self.adbshell('ime', 'enable', adbime)
        self.adbshell('ime', 'set', adbime)
        first = True
        for s in text.split('\n'):
            if first:
                first=False
            else:
                KEYCODE_ENTER = '66'
                self.adbshell('am', 'broadcast', '-a', 'ADB_INPUT_CODE', '--ei', 'code', KEYCODE_ENTER)
            if not s:
                continue
            self.adbshell('am', 'broadcast', '-a', 'ADB_INPUT_TEXT', '--es', 'msg', s)
        self.adbshell('ime', 'disable', adbime)

    def keyevent(self, event):
        '''
        Send keyevent by adb

        @param event: string (one of MENU, HOME, BACK)
        '''
        self.adbshell('input', 'keyevent', str(event))

    def start_app(self, appname, activity):
        '''
        Start a program

        @param extra: dict (defined in air.json)
        '''
        self.adbshell('am', 'start', '-n', appname+'/'+activity)

    def stop_app(self, appname):
        '''
        Stop app
        '''
        self.adbshell('am', 'force-stop', appname)
    #
    # ------------ useless below -------------------
    #
    # def meminfo(self, appname):
    #     '''
    #     Retrive memory info for app
    #     @param package(string): android package name
    #     @return dict: {'VSS', 'RSS', 'PSS'} (unit KB)
    #     '''
    #     return _get_meminfo(self._serialno, appname)

    # def cpuinfo(self, appname):
    #     '''
    #     @param package(string): android package name
    #     @return dict: {'total': float, 'average': float}
    #     '''
    #     total = _get_cpuinfo(self._serialno, appname)
    #     ncpu=self._devinfo['cpu_count']
    #     return total/ncpu #dict(total=total, )


    # def clear(self, appname):
    #     '''
    #     Stop app and clear data
    #     '''
    #     self.adbshell('pm', 'clear', appname)

    # def getdevinfo(self):
    #     # cpu
    #     output = self.adbshell('cat', '/proc/cpuinfo')
    #     matches = re.compile('processor').findall(output)
    #     cpu_count = len(matches)
    #     # mem
    #     output = self.adbshell('cat', '/proc/meminfo')
    #     match = re.compile('MemTotal:\s*(\d+)\s*kB\s*MemFree:\s*(\d+)', re.IGNORECASE).match(output)
    #     if match:
    #         mem_total = int(match.group(1), 10)>>10 # MB
    #         mem_free = int(match.group(2), 10)>>10
    #     else:
    #         mem_total = -1
    #         mem_free = -1

    #     # brand = self.adb.getProperty('ro.product.brand')
    #     return {
    #         'cpu_count': cpu_count,
    #         'mem_total': mem_total,
    #         'mem_free': mem_free,
    #         'product_brand': self.adbclient.getProperty('ro.product.brand'),
    #         'product_model': self.adbclient.getProperty('ro.product.model')
    #         }



#-*- encoding: utf-8 -*-

import os
import time

from atx.device.windows import Window, FrozenWindow, WindowsDevice


# def _input_left_mouse(self, x, y):
#     left, top, right, bottom = self.position
#     width, height = right - left, bottom - top
#     if x < 0 or x > width or y < 0 or y > height:
#         return

#     win32gui.SetForegroundWindow(self._handle)
#     pos = win32gui.GetCursorPos()
#     win32api.SetCursorPos((left+x, top+y))
#     win32api.mouse_event(win32con.MOUSEEVENTF_LEFTDOWN, 0, 0)
#     win32api.Sleep(100) #ms
#     win32api.mouse_event(win32con.MOUSEEVENTF_LEFTUP, 0, 0)
#     win32api.Sleep(100) #ms
#     win32api.SetCursorPos(pos)

# def drag(self):
#     pass

# def _input_keyboard(self, text):
#     pass


def test():
    try:
        name = u"Windows 任务管理器"
        win = FrozenWindow(name.encode("gbk"), exclude_border=True)
        win.set_foreground()
        time.sleep(0.1)
        win._screenshot('taskman-pil.png')
        time.sleep(0.5)
        win._screenshot_cv2('taskman-cv2.png')
    except Exception as e:
        print str(e)

    try:
        filepath = "C:\\Windows\\System32\\calc.exe"
        win = Window(exe_file=filepath)
        win.set_foreground()
        time.sleep(0.1)
        win._screenshot('calc-pil.png')
        time.sleep(0.5)
        win._screenshot_cv2('calc-cv2.png')
    except Exception as e:
        print str(e)

    dev = WindowsDevice()
    dev.screenshot('screen.png')

if __name__ == '__main__':
    test() 


# coding: utf-8
#
# Insert code into files
#

def insert_code(filename, code, save=True, marker='# ATX CODE'):
    content = ''
    for line in open(filename, 'rb'):
        if line.strip() == marker:
            cnt = line.find(marker)
            content += line[:cnt] + code
        content += line
    if save:
        with open(filename, 'wb') as f:
            f.write(content)
    return content

if __name__ == '__main__':
    insert_code('README.md', 'hello world\n')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

SCREENSHOT_METHOD_UIAUTOMATOR = 'uiautomator'
SCREENSHOT_METHOD_MINICAP = 'minicap'
SCREENSHOT_METHOD_AUTO = 'auto'

IMAGE_MATCH_METHOD_TMPL = 'template'
IMAGE_MATCH_METHOD_SIFT = 'sift'

EVENT_UIAUTO_TOUCH = 1 << 0
EVENT_UIAUTO_CLICK = 1 << 0 # alias for touch
EVENT_UIAUTO_SWIPE = 1 << 2

EVENT_SCREENSHOT = 1 << 3
EVENT_CLICK = 1 << 4
EVENT_CLICK_IMAGE = 1 << 5

EVENT_ALL = EVENT_SCREENSHOT | EVENT_CLICK
# 1 - 2 - 3
# 4 - 5 - 6
# 7 - 8 - 9

NW = 1
N = 2
NE = 3
W = 4
E = 6
SW = 7
S = 8
SE = 9



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# License under MIT


from __future__ import absolute_import

import collections
import os
import re
import sys
import subprocess
import time
import tempfile
import warnings
import logging
import xml.dom.minidom

from uiautomator import Device as UiaDevice
from uiautomator import AutomatorDeviceObject
from PIL import Image

from atx import consts
from atx import errors
from atx import patch
from atx import base
from atx import imutils
from atx import adb
from atx import strutils
from atx.device import Bounds
from atx import logutils
from atx.device.mixin import DeviceMixin, hook_wrap


DISPLAY_RE = re.compile(
    r'.*DisplayViewport{valid=true, .*orientation=(?P<orientation>\d+), .*deviceWidth=(?P<width>\d+), deviceHeight=(?P<height>\d+).*')

PROP_PATTERN = re.compile(
    r'\[(?P<key>.*?)\]:\s*\[(?P<value>.*)\]')

UINode = collections.namedtuple('UINode', [
    'xml',
    'bounds', 
    'selected', 'checkable', 'clickable', 'scrollable', 'focusable', 'enabled', 'focused', 'long_clickable',
    'password',
    'class_name',
    'index', 'resource_id',
    'text', 'content_desc',
    'package'])

log = logutils.getLogger(__name__)


def getenv(name, default_value=None, type=str):
    value = os.getenv(name)
    return type(value) if value else default_value

class AndroidMixin(object):
    def __init__(self, serialno=None):
        pass

class AndroidDevice(DeviceMixin, UiaDevice):
    def __init__(self, serialno=None, **kwargs):
        """Initial AndroidDevice
        Args:
            serialno: string specify which device

        Returns:
            AndroidDevice object

        Raises:
            EnvironmentError
        """
        serialno = serialno or getenv('ATX_ADB_SERIALNO', None)
        self._host = kwargs.get('host', getenv('ATX_ADB_HOST', '127.0.0.1'))
        self._port = kwargs.get('port', getenv('ATX_ADB_PORT', 5037, type=int))
        self._adb = adb.Adb(serialno, self._host, self._port)
        serialno = self._adb.device_serial()

        kwargs['adb_server_host'] = kwargs.pop('host', self._host)
        kwargs['adb_server_port'] = kwargs.pop('port', self._port)
        UiaDevice.__init__(self, serialno, **kwargs)
        DeviceMixin.__init__(self)

        self._randid = base.id_generator(5)
        self._serial = serialno
        self._uiauto = super(AndroidDevice, self)

        self.screen_rotation = None
        self.screenshot_method = consts.SCREENSHOT_METHOD_AUTO
        self.last_screenshot = None

    @property
    def serial(self):
        """ Android Device Serial Number """
        return self._serial
    
    @property
    def wlan_ip(self):
        """ Wlan IP """
        return self.adb_shell(['getprop', 'dhcp.wlan0.ipaddress']).strip()

    def forward(self, device_port, local_port=None):
        """Forward device port to local
        Args:
            device_port: port inside device
            local_port: port on PC, if this value is None, a port will random pick one.

        Returns:
            tuple, (host, local_port)
        """
        port = self._adb.forward(device_port, local_port)
        return (self._host, port)

    @property
    def current_package_name(self):
        return self.info['currentPackageName']

    def is_app_alive(self, package_name):
        """ Deprecated: use current_package_name instaed.
        Check if app in running in foreground """
        return self.info['currentPackageName'] == package_name

    def sleep(self, secs=None):
        """Depreciated. use delay instead."""
        if secs is None:
            self._uiauto.sleep()
        else:
            self.delay(secs)

    @property
    @patch.run_once
    def display(self):
        """Virtual keyborad may get small d.info['displayHeight']
        """
        w, h = (0, 0)
        for line in self.adb_shell('dumpsys display').splitlines():
            m = DISPLAY_RE.search(line, 0)
            if not m:
                continue
            w = int(m.group('width'))
            h = int(m.group('height'))
            # o = int(m.group('orientation'))
            w, h = min(w, h), max(w, h)
            return collections.namedtuple('Display', ['width', 'height'])(w, h)

        w, h = self.info['displayWidth'], self.info['displayHeight']
        w, h = min(w, h), max(w, h)
        return collections.namedtuple('Display', ['width', 'height'])(w, h)
    
    @property
    def rotation(self):
        """
        Rotaion of the phone

        0: normal
        1: home key on the right
        2: home key on the top
        3: home key on the left
        """
        if self.screen_rotation in range(4):
            return self.screen_rotation
        return self.info['displayRotation']
    
    def _minicap_params(self):
        """
        Used about 0.1s
        uiautomator d.info is now well working with device which has virtual menu.
        """
        rotation = self.screen_rotation 
        if self.screen_rotation is None:
            rotation = self.rotation

        # rotation not working on SumSUNG 9502
        return '{x}x{y}@{x}x{y}/{r}'.format(
            x=self.display.width,
            y=self.display.height,
            r=rotation*90)
    
    def _screenshot_minicap(self):
        phone_tmp_file = '/data/local/tmp/_atx_screen-{}.jpg'.format(self._randid)
        local_tmp_file = tempfile.mktemp(prefix='atx-tmp-', suffix='.jpg')
        command = 'LD_LIBRARY_PATH=/data/local/tmp /data/local/tmp/minicap -P {} -s > {}'.format(
            self._minicap_params(), phone_tmp_file)
        try:
            self.adb_shell(command)
            self.adb_cmd(['pull', phone_tmp_file, local_tmp_file])
            image = imutils.open_as_pillow(local_tmp_file)

            # Fix rotation not rotate right.
            (width, height) = image.size
            if self.screen_rotation in [1, 3] and width < height:
                image = image.rotate(90, Image.BILINEAR, expand=True)
            return image
        except IOError:
            raise IOError("Screenshot use minicap failed.")
        finally:
            # remove_force(local_tmp_file)
            self.adb_shell(['rm', phone_tmp_file])

    def _screenshot_uiauto(self):
        tmp_file = tempfile.mktemp(prefix='atx-tmp-', suffix='.jpg')
        self._uiauto.screenshot(tmp_file)
        try:
            return imutils.open_as_pillow(tmp_file)
        except IOError:
            raise IOError("Screenshot use uiautomator failed.")
        finally:
            base.remove_force(tmp_file)

    @hook_wrap(consts.EVENT_CLICK)
    def click(self, x, y):
        """
        Touch specify position

        Args:
            x, y: int

        Returns:
            None
        """
        return self._uiauto.click(x, y)

    @hook_wrap(consts.EVENT_SCREENSHOT)
    def screenshot(self, filename=None):
        """
        Take screen snapshot

        Args:
            filename: filename where save to, optional

        Returns:
            PIL.Image object

        Raises:
            TypeError, IOError
        """
        screen = None
        if self.screenshot_method == consts.SCREENSHOT_METHOD_UIAUTOMATOR:
            screen = self._screenshot_uiauto()
        elif self.screenshot_method == consts.SCREENSHOT_METHOD_MINICAP:
            screen = self._screenshot_minicap()
        elif self.screenshot_method == consts.SCREENSHOT_METHOD_AUTO:
            try:
                screen = self._screenshot_minicap()
                self.screenshot_method = consts.SCREENSHOT_METHOD_MINICAP
            except IOError:
                screen = self._screenshot_uiauto()
                self.screenshot_method = consts.SCREENSHOT_METHOD_UIAUTOMATOR
        else:
            raise TypeError('Invalid screenshot_method')

        if filename:
            save_dir = os.path.dirname(filename) or '.'
            if not os.path.exists(save_dir):
                os.makedirs(save_dir)
            screen.save(filename)

        self.last_screenshot = screen
        return screen

    def adb_cmd(self, command):
        '''
        Run adb command, for example: adb(['pull', '/data/local/tmp/a.png'])

        Args:
            command: string or list of string

        Returns:
            command output
        '''
        cmds = ['adb']
        if self._serial:
            cmds.extend(['-s', self._serial])
        cmds.extend(['-H', self._host, '-P', str(self._port)])

        if isinstance(command, list) or isinstance(command, tuple):
            cmds.extend(list(command))
        else:
            cmds.append(command)
        # print cmds
        output = subprocess.check_output(cmds, stderr=subprocess.STDOUT)
        return output.replace('\r\n', '\n')

    def adb_shell(self, command):
        '''
        Run adb shell command

        Args:
            command: string or list of string

        Returns:
            command output
        '''
        if isinstance(command, list) or isinstance(command, tuple):
            return self.adb_cmd(['shell'] + list(command))
        else:
            return self.adb_cmd(['shell'] + [command])

    @property
    def properties(self):
        '''
        Android Properties, extracted from `adb shell getprop`

        Returns:
            dict of props, for
            example:

                {'ro.bluetooth.dun': 'true'}
        '''
        props = {}
        for line in self.adb_shell(['getprop']).splitlines():
            m = PROP_PATTERN.match(line)
            if m:
                props[m.group('key')] = m.group('value')
        return props

    def start_app(self, package_name):
        '''
        Start application

        Args:
            package_name: string like com.example.app1

        Returns:
            None
        '''
        self.adb_shell(['monkey', '-p', package_name, '-c', 'android.intent.category.LAUNCHER', '1'])
        return self

    def stop_app(self, package_name, clear=False):
        '''
        Stop application

        Args:
            package_name: string like com.example.app1
            clear: bool, remove user data

        Returns:
            None
        '''
        if clear:
            self.adb_shell(['pm', 'clear', package_name])
        else:
            self.adb_shell(['am', 'force-stop', package_name])
        return self

    def takeSnapshot(self, filename):
        '''
        Deprecated, use screenshot instead.
        '''
        warnings.warn("deprecated, use snapshot instead", DeprecationWarning)
        return self.screenshot(filename)

    def _parse_xml_node(self, node):
        # ['bounds', 'checkable', 'class', 'text', 'resource_id', 'package']
        __alias = {
            'class': 'class_name',
            'resource-id': 'resource_id',
            'content-desc': 'content_desc',
            'long-clickable': 'long_clickable',
        }

        def parse_bounds(text):
            m = re.match(r'\[(\d+),(\d+)\]\[(\d+),(\d+)\]', text)
            if m is None:
                return None
            return Bounds(*map(int, m.groups()))

        def str2bool(v):
            return v.lower() in ("yes", "true", "t", "1")

        def convstr(v):
            return v.encode('utf-8')

        parsers = {
            'bounds': parse_bounds,
            'text': convstr,
            'class_name': convstr,
            'resource_id': convstr,
            'package': convstr,
            'checkable': str2bool,
            'scrollable': str2bool,
            'focused': str2bool,
            'clickable': str2bool,
            'enabled': str2bool,
            'selected': str2bool,
            'long_clickable': str2bool,
            'focusable': str2bool,
            'password': str2bool,
            'index': int,
            'content_desc': convstr,
        }
        ks = {}
        for key, value in node.attributes.items():
            key = __alias.get(key, key)
            f = parsers.get(key)
            if value is None:
                ks[key] = None
            elif f:
                ks[key] = f(value)
        for key in parsers.keys():
            ks[key] = ks.get(key)
        ks['xml'] = node

        return UINode(**ks)

    def dump_nodes(self):
        """Dump current screen UI to list
        Returns:
            List of UINode object, For
            example:

            [UINode(
                bounds=Bounds(left=0, top=0, right=480, bottom=168),
                checkable=False,
                class_name='android.view.View',
                text='',
                resource_id='',
                package='com.sonyericsson.advancedwidget.clock')]
        """
        xmldata = self._uiauto.dump()
        dom = xml.dom.minidom.parseString(xmldata.encode('utf-8'))
        root = dom.documentElement
        nodes = root.getElementsByTagName('node')
        ui_nodes = []
        for node in nodes:
            ui_nodes.append(self._parse_xml_node(node))
        return ui_nodes

    def _escape_text(self, s):
        return s.replace(' ', '%s')

    def keyevent(self, keycode):
        """call adb shell input keyevent ${keycode}

        Args:
            - keycode(string): for example, KEYCODE_ENTER

        keycode need reference:
        http://developer.android.com/reference/android/view/KeyEvent.html
        """
        self.adb_shell(['input', 'keyevent', keycode])

    def type(self, text, enter=False):
        """Input some text, this method has been tested not very stable on some device.
        "Hi world" maybe spell into "H iworld"

        Args:
            - text: string (text to input)
            - enter(bool): input enter at last

        The android source code show that
        space need to change to %s
        insteresting thing is that if want to input %s, it is really unconvinent.
        android source code can be found here.
        https://android.googlesource.com/platform/frameworks/base/+/android-4.4.2_r1/cmds/input/src/com/android/commands/input/Input.java#159
        """
        first = True
        for s in text.split('%s'):
            if s == '':
                continue
            estext = self._escape_text(s)
            if first:
                first = False
            else:
                self.adb_shell(['input', 'text', '%'])
                estext = 's' + estext
            self.adb_shell(['input', 'text', estext])

        if enter:
            self.keyevent('KEYCODE_ENTER')



#-*- encoding: utf-8 -*-

# import win32api
# import win32con
import win32gui
# import win32process
import pywintypes
from pyHook import HookManager, HookConstants

from atx.record.base import BaseRecorder

class WindowsRecorder(BaseRecorder):

    KBFLAG_CTRL = 0x01
    KBFLAG_ALT = 0x02
    KBFLAG_SHIFT = 0x04
    KBFLAG_CAPS = 0x08

    default_radius = 25
    capture_interval = 0.05
    capture_maxnum = 30

    def __init__(self, device=None):
        self.watched_hwnds = set()
        super(WindowsRecorder, self).__init__(device)
        self.kbflag = 0
        self.hm = HookManager()
        self.hm.MouseAllButtons = self._hook_on_mouse
        self.hm.KeyAll = self._hook_on_keyboard

    def attach(self, device):
        if self.device is not None:
            print "Warning: already attached to a device."
            if device is not self.device:
                self.detach()

        handle = device.hwnd
        def callback(hwnd, extra):
            extra.add(hwnd)
            return True
        self.watched_hwnds.add(handle)
        try:
            # EnumChildWindows may crash for windows have no any child.
            # refs: https://mail.python.org/pipermail/python-win32/2005-March/003042.html
            win32gui.EnumChildWindows(handle, callback, self.watched_hwnds)
        except pywintypes.error:
            pass

        self.device = device
        print "attach to device", device

    def detach(self):
        print "detach from device", self.device
        self.device = None
        self.watched_hwnds = set()

    def hook(self):
        self.hm.HookMouse()
        self.hm.HookKeyboard()

    def unhook(self):
        self.hm.UnhookMouse()
        self.hm.UnhookKeyboard()

    def _hook_on_mouse(self, event):
        if self.device is None:
            return True
        if event.Window not in self.watched_hwnds:
            return True
        if event.Message == HookConstants.WM_LBUTTONUP:
            x, y = self.device.norm_position(event.Position)
            # ignore the clicks outside the rect if the window has a frame.
            if x < 0 or y < 0:
                return True
            self.on_click((x, y))

        return True

    def _hook_on_keyboard(self, event):
        if self.device is None:
            return True
        if event.Window not in self.watched_hwnds:
            return True
        print "on_keyboard", event.MessageName, event.Key, repr(event.Ascii), event.KeyID, event.ScanCode, 
        print event.flags, event.Extended, event.Injected, event.Alt, event.Transition
        return True


#!/usr/bin/env python

# This file is part of Androguard.
#
# Copyright (C) 2012, Anthony Desnos <desnos at t0t0.fr>
# All rights reserved.
#
# Androguard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Androguard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Androguard.  If not, see <http://www.gnu.org/licenses/>.

import os
__dir__ = os.path.dirname(os.path.abspath(__file__))

import sys
sys.path.append(os.path.join(__dir__, "androguard.zip"))

from androguard.core.bytecodes import apk


def _xml2parse(dom): #, strformat='$package/$activity'):
    root = dom.getElementsByTagName("manifest")[0]
    package = root.getAttribute('package')
    activity = ''
    for e in root.getElementsByTagName('activity'):
        name = e.getAttribute('android:name')
        t = e.getElementsByTagName('intent-filter')
        action = t and t[0].getElementsByTagName('action')
        category = t and t[0].getElementsByTagName('category')
        if action and action[0].getAttribute('android:name') == 'android.intent.action.MAIN' and \
                category and category[0].getAttribute('android:name') == 'android.intent.category.LAUNCHER':
            activity = name
    return (package, activity)

def parse_apk(filename):
    ''' return (package, activity) '''
    a = apk.APK(filename)
    dom = a.get_android_manifest_xml()
    return _xml2parse(dom)




#-*- encoding: utf-8 -*-

import os
import sys
import win32api
import win32con
import win32gui
import traceback
import Tkinter as tk

from atx.device.android import AndroidDevice
from atx.device.android_minicap import AndroidDeviceMinicap
from atx.device.windows import WindowsDevice
from atx.record.android import AndroidRecorder
from atx.record.windows import WindowsRecorder

__dir__ = os.path.dirname(os.path.abspath(__file__))

class SystemTray(object):
    def __init__(self, parent, name, commands=None, icon_path=None):
        self.parent = parent
        self.name = name
        self.WM_NOTIFY = win32con.WM_USER+20

        wndproc = {
            win32con.WM_DESTROY: self.on_destroy,
            win32con.WM_COMMAND: self.on_command,
            self.WM_NOTIFY: self.on_tray_notify,
        }

        wc = win32gui.WNDCLASS()
        wc.hInstance = hinst = win32api.GetModuleHandle(None)
        wc.lpszClassName = name.title()
        wc.lpfnWndProc = wndproc
        class_atom = win32gui.RegisterClass(wc)
        self.hwnd = win32gui.CreateWindow(wc.lpszClassName, "", win32con.WS_POPUP, 0,0,1,1, parent, 0, hinst, None)
        win32gui.UpdateWindow(self.hwnd)

        if icon_path is not None and os.path.isfile(icon_path):
            icon_flags = win32con.LR_LOADFROMFILE | win32con.LR_DEFAULTSIZE
            hicon = win32gui.LoadImage(None, icon_path, win32con.IMAGE_ICON, 0, 0, icon_flags)
        else:
            shell_dll = os.path.join(win32api.GetSystemDirectory(), "shell32.dll")
            large, small = win32gui.ExtractIconEx(shell_dll, 19, 1) #19 or 76
            hicon = small[0]
            win32gui.DestroyIcon(large[0])
        self.hicon = hicon

        flags = win32gui.NIF_ICON | win32gui.NIF_MESSAGE | win32gui.NIF_TIP | win32gui.NIF_INFO
        nid = (self.hwnd, 0, flags, self.WM_NOTIFY, self.hicon, self.name)
        win32gui.Shell_NotifyIcon(win32gui.NIM_ADD, nid)

        self.next_command_id = 1000
        self.commands = {}
        self.register_command('Exit', lambda:win32gui.DestroyWindow(self.hwnd))
        if commands is not None:
            for n, f in commands[::-1]:
                self.register_command(n, f)

    def balloon(self, msg, title=""):
        flags = win32gui.NIF_ICON | win32gui.NIF_MESSAGE | win32gui.NIF_TIP | win32gui.NIF_INFO
        nid = (self.hwnd, 0, flags, self.WM_NOTIFY, self.hicon, self.name, msg, 300, title)
        win32gui.Shell_NotifyIcon(win32gui.NIM_MODIFY, nid)

    def register_command(self, name, func):
        cid = self.next_command_id
        self.next_command_id += 1
        self.commands[cid] = (name, func)
        return cid

    def on_destroy(self, hwnd, msg, wp, lp):
        win32gui.Shell_NotifyIcon(win32gui.NIM_DELETE, (self.hwnd, 0))
        win32gui.PostMessage(self.parent, win32con.WM_CLOSE, 0, 0)
        return True

    def on_command(self, hwnd, msg, wp, lp):
        cid = win32api.LOWORD(wp)
        if not self.commands.get(cid):
            print "Unknown command -", cid
            return
        _, func = self.commands[cid]
        try:
            func()
        except Exception as e:
            traceback.print_exc()
        return True

    def on_tray_notify(self, hwnd, msg, wp, lp):
        if lp == win32con.WM_LBUTTONUP:
            # print "left click"
            # win32gui.SetForegroundWindow(self.hwnd)
            pass
        elif lp == win32con.WM_RBUTTONUP:
            # print "right click"
            menu = win32gui.CreatePopupMenu()
            for cid in sorted(self.commands.keys(), reverse=True):
                name, _ = self.commands[cid]
                win32gui.AppendMenu(menu, win32con.MF_STRING, cid, name)

            pos = win32gui.GetCursorPos()
            win32gui.SetForegroundWindow(self.hwnd)
            win32gui.TrackPopupMenu(menu, win32con.TPM_LEFTALIGN, pos[0], pos[1], 0, self.hwnd, None)
            win32gui.PostMessage(self.hwnd, win32con.WM_NULL, 0, 0)
        return True

class RecorderGUI(object):
    def __init__(self, device=None):
        self._device = device
        self._recorder = None
        self._root = root = tk.Tk()

        root.protocol("WM_DELETE_WINDOW", self.destroy)

        def calllater():
            icon_path = os.path.join(__dir__, 'static', 'recorder.ico')
            commands  = [
                ('Choose Device', self.choose_device),
                ("Start Record", self.start_record),
                ("Stop Record", self.stop_record),
            ]
            tray = SystemTray(root.winfo_id(), "recorder", commands, icon_path)
            tray.balloon('hello')

        root.after(1000, calllater)

        # no window for now.
        root.withdraw()

        # need to handle device event

    def destroy(self):
        print "root destroy"
        if self._recorder is not None:
            self._recorder.stop()
        self._root.destroy()
        win32api.PostQuitMessage(0)

    def mainloop(self):
        try:
            self._root.mainloop()
        except KeyboardInterrupt:
            self.destroy()

    def choose_device(self):
        pass

    def start_record(self):
        if not self.check_recorder():
            return
        self._recorder.run()

    def stop_record(self):
        if not self.check_recorder():
            return
        self._recorder.stop()

    def check_recorder(self):
        if self._device is None:
            print "No device choosen."
            self._recorder = None
            return False

        if self._recorder is None:
            print "init recorder", type(self._device)
            if isinstance(self._device, WindowsDevice):
                record_class = WindowsRecorder
            elif isinstance(self._device, (AndroidDevice, AndroidDeviceMinicap)):
                record_class = AndroidRecorder
            else:
                print "Unknown device type", type(self._device)
                return False
            self._recorder = record_class(self._device)
        return True

def main():
    w = RecorderGUI()
    w.mainloop()  

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import subprocess32 as subprocess


class IOS(object):
    @classmethod
    def cmd(cls, *args):
        return subprocess.Popen(list(args), stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    @classmethod
    def devices(cls):
        p = cls.cmd('idevice_id', '-l')
        return p.communicate()[0].strip().split()


if __name__ == '__main__':
    print IOS.devices()



# coding: utf-8
import cv2

method = cv2.TM_CCORR_NORMED
method = cv2.TM_SQDIFF_NORMED

def locate_img(image, template):
    img = image.copy()
    res = cv2.matchTemplate(img, template, method)
    print res
    print res.shape
    cv2.imwrite('image/shape.png', res)
    min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(res)
    print cv2.minMaxLoc(res)
    if method in [cv2.TM_SQDIFF, cv2.TM_SQDIFF_NORMED]:
        top_left = min_loc
    else:
        top_left = max_loc
    h, w = template.shape
    bottom_right = (top_left[0] + w, top_left[1]+h)
    cv2.rectangle(img, top_left, bottom_right, 255, 2)
    cv2.imwrite('image/tt.jpg', img)

image = cv2.imread('image/mule_mobile.png', 0)
#image = cv2.imread('image/mule.png', 0)
template = cv2.imread('image/template.png', 0)
locate_img(image, template)



# -*- coding: utf-8 -*-
# An android event hook via getevent.
# Only ABS_MT_POSITION_X(Y) events are handled.
#
# Basic input: TouchDown(D), TouchUp(U), TouchMove(M)
# Basic timeouts: TouchPressTimeout(P), TouchFollowTimeout(F), TouchMoveStopTimeout(S)
# guestures are defined as follows:
#   Tap: DM?UF
#   TapFollow: (DM?U)+DM?UF
#   LongPress: DP, may be followed by Drag or half-Fling
#   Drag: D?M+S, may be followed by Drag or half-Fling
#   Fling: DM+U
#   2-Finger-Pinch: distance changing
#   2-Finger-Drag: distance hold while moving
# where '?' after M means a little movement and '+' means a large one.
# other guestures are ignored. 

import re
import math
import time
import numpy as np
import subprocess
import threading
import Queue
import traceback

__all__ = ['AndroidInputHookManager', 'HookManager', 'HookConstants']

class HookConstants:
    # basic events
    TOUCH_DOWN = 0x01
    TOUCH_UP   = 0x02
    TOUCH_MOVE = 0x03
    TOUCH_PRESS_TIMEOUT = 0x04
    TOUCH_FOLLOW_TIMEOUT = 0x05
    TOUCH_MOVESTOP_TIMEOUT = 0x06

    KEY_ANY = 0x10
    KEY_HOME_DOWN = 0x11
    KEY_HOME_UP   = 0x12
    KEY_BACK_DOWN = 0x13
    KEY_BACK_UP   = 0x14
    KEY_MENU_DOWN = 0x15
    KEY_MENU_UP   = 0x16
    KEY_POWER_DOWN      = 0x17
    KEY_POWER_UP        = 0x18
    KEY_VOLUMEDOWN_DOWN = 0x19
    KEY_VOLUMEDOWN_UP   = 0x1a
    KEY_VOLUMEUP_DOWN   = 0x1b
    KEY_VOLUMEUP_UP     = 0x1c

    # gestures
    GST_TAP = 0x21
    GST_DOUBLE_TAP = 0x22
    GST_LONG_PRESS = 0x23
    GST_SWIPE = 0x24
    GST_DRAG = 0x25
    GST_PINCH = 0x26

    # maybe used for uiautomator press
    KEY_HOME = 'home'
    KEY_MENU = 'menu'
    KEY_BACK = 'back'
    KEY_POWER      = 'power'
    KEY_VOLUMEDOWN = 'volume_down'
    KEY_VOLUMEUP   = 'volume_up'

class Event(object):
    msg = None
    def __init__(self, time, slotid=None):
        self.time = time
        self.slotid = slotid

class TouchEvent(Event):
    def __init__(self, time, slotid, x, y, pressure, touch_major):
        '''msg: touch_down, touch_up, touch_move'''
        super(TouchEvent, self).__init__(time, slotid)
        self.x = x
        self.y = y
        self.pressure = pressure
        self.touch_major = touch_major

class TouchDownEvent(TouchEvent):
    msg = HookConstants.TOUCH_DOWN

class TouchUpEvent(TouchEvent):
    msg = HookConstants.TOUCH_UP

class TouchMoveEvent(TouchEvent):
    msg = HookConstants.TOUCH_MOVE
    def __init__(self, time, slotid, x, y, pressure, touch_major, distance, direction, speed):
        super(TouchMoveEvent, self).__init__(time, slotid, x, y, pressure, touch_major)
        self.distance = distance
        self.direction = direction
        self.speed = speed

class TouchPressTimeout(Event):
    msg = HookConstants.TOUCH_PRESS_TIMEOUT

class TouchFollowTimeout(Event):
    msg = HookConstants.TOUCH_FOLLOW_TIMEOUT

class TouchMoveStopTimeout(Event):
    msg = HookConstants.TOUCH_MOVESTOP_TIMEOUT

class KeyEvent(Event):
    msg = HookConstants.KEY_ANY
    def __init__(self, time, msg, key):
        '''msg: keydown, keyup'''
        super(KeyEvent, self).__init__(time)
        self.key = key
        self.msg = msg

class GestureEvent(Event):
    msg = None
    def __init__(self, time, endtime, slotid):
        super(GestureEvent, self).__init__(time, slotid)
        self.endtime = endtime

class Tap(GestureEvent):
    msg = HookConstants.GST_TAP

class DoubleTap(GestureEvent):
    msg = HookConstants.GST_DOUBLE_TAP


SLOT_NUM = 5
_X, _Y, _VR, _VA, _MJ, _PR, FIELD_NUM = range(7)
INF = 9999

class InputParser(object):
    _pat = re.compile('\[\s*(?P<time>[0-9.]+)\] (?P<device>/dev/.*): +(?P<type>\w+) +(?P<code>\w+) +(?P<value>\w+)')
    _move_radius = 10

    def __init__(self, queue):
        self.queue = queue
        # the 'standard' status temp_status is compared to. 
        # if changes are great enough, new event are emitted.
        # velocity will be calculated for touch-move events.
        self._status = np.ones((SLOT_NUM, FIELD_NUM), dtype=int) * (-INF)
        self._status_time = 0
        # realtime status, minor changes are cumulated
        self._temp_status = np.ones((SLOT_NUM, FIELD_NUM), dtype=int) * (-INF)
        self._temp_status_time = 0
        # self._trackids = [None] * SLOT_NUM

        self._touch_batch = []
        self._curr_slot = 0

    def feed(self, line):
        # print line
        m = self._pat.search(line)
        if not m:
            return
        _time, _device, _type, _code, _value = m.groups()
        _time = float(_time)
        try:
            _value = int(_value, 16)
        except:
            pass

        if _type == 'EV_SYN':
            if _code in ('SYN_REPORT', 'SYN_MT_REPORT'):
                self._process_touch_batch()
            elif _code == 'SYN_DROPPED':
                self._touch_batch = []
            else:
                print 'unknown syn code', _code
        elif _type == 'EV_KEY':
            self.emit_key_event(_time, _code, _value)
        elif _type == 'EV_ABS':
            self._touch_batch.append((_time, _device, _type, _code, _value))
        else:
            print 'unknown input event type', _type

    def emit_key_event(self, _time, _code, _value):
        name = '%s_%s' % (_code, _value)
        msg = getattr(HookConstants, name, None)
        if msg is None:
            return
        event = KeyEvent(_time, msg, _code)
        self.queue.put(event)

    def emit_touch_event(self, event):
        self.queue.put(event)

    def _process_touch_batch(self):
        '''a batch syncs in about 0.001 seconds.'''
        if not self._touch_batch:
            return

        _time = self._temp_status_time
        changed = False

        for (_time, _device, _type, _code, _value) in self._touch_batch:
            if _code == 'ABS_MT_TRACKING_ID':
                if _value == 0xffffffff:
                    # self._trackids[self._curr_slot] = None
                    self._temp_status[self._curr_slot] = -INF
                    changed = True
                else:
                    # self._trackids[self._curr_slot] = _value
                    pass
            elif _code == 'ABS_MT_SLOT':
                self._curr_slot = _value
            else:
                if _code == 'ABS_MT_POSITION_X':
                    self._temp_status[self._curr_slot,_X] = _value
                    changed = True
                elif _code == 'ABS_MT_POSITION_Y':
                    self._temp_status[self._curr_slot,_Y] = _value
                    changed = True
                elif _code == 'ABS_MT_PRESSURE':
                    self._temp_status[self._curr_slot,_PR] = _value
                elif _code == 'ABS_MT_TOUCH_MAJOR':
                    self._temp_status[self._curr_slot,_MJ] = _value
                else:
                    print 'Unknown code', _code

        self._temp_status_time = _time
        self._touch_batch = []
        if not changed:
            return

        # check differences, if position changes are big enough then emit events
        diff = self._temp_status - self._status
        dt = self._temp_status_time - self._status_time
        emitted = False
        for i in range(SLOT_NUM):
            arr = self._temp_status[i]
            oldarr = self._status[i]
            # trackid = self._trackids[i]
            dx, dy = diff[i,_X], diff[i,_Y]
            if dx > INF or dy > INF:
                # touch begin
                event = TouchDownEvent(_time, i, arr[_X], arr[_Y], arr[_PR], arr[_MJ])
                self.emit_touch_event(event)
                emitted = True
            elif dx < -INF or dy < -INF:
                # touch end
                event = TouchUpEvent(_time, i, oldarr[_X], oldarr[_Y], oldarr[_PR], oldarr[_MJ])
                self.emit_touch_event(event)
                emitted = True
            else:
                r, a = radang(float(dx), float(dy))
                if r > self._move_radius:
                    v = r / dt
                    event = TouchMoveEvent(_time, i, arr[_X], arr[_Y], arr[_PR], arr[_MJ], r, a, v)
                    self.emit_touch_event(event)
                    emitted = True

        if not emitted:
            return
        self._status = self._temp_status.copy()
        self._status_time = self._temp_status_time

def radang(x, y):
    '''return (radius, angle) of a vector(x, y)'''
    if x == 0:
        if y == 0:
            return 0, 0
        return abs(y), 90+180*(y<0)
    if y == 0:
        return abs(x), 180*(x<0)

    r = math.sqrt(x*x+y*y)
    a = math.degrees(math.atan(y/x))
    if x < 0:
        a += 180
    elif y < 0: 
        a += 360
    return r, a

class GestureRecognizer(object):

    double_tap_delay = 0.5
    long_press_delay = 1
    move_stop_delay = 0.5
    pinch_difference_square = 3000

    def __init__(self, queue):
        self.queue = queue
        self.dispatch_map = {}
        self.running = False

        # only used in process thread so we don't need a lock
        self.touchdowns = [None]*SLOT_NUM
        self.touchups = [None]*SLOT_NUM
        self.touchmoves = [None]*SLOT_NUM

        # used for recognition
        self.tracks = [[] for i in range(SLOT_NUM)]
        self.track_slots = set()

    def handle_event(self, event):
        print 'handle event', event
        self.dispatch_event(event)
        if event.msg & HookConstants.KEY_ANY: # skip keys
            return
        self.analyze_tracks(event)

    def analyze_tracks(self, e):
        # handle one-finger and two-finger gestures only
        # means a third finger will be ignored even if one of the
        # first two fingers leaves the screen.
        # hint: try it on <Boom Beach>

        HC = HookConstants
        i = e.slotid

        # end guesture when touch up
        if e.msg == HC.TOUCH_UP:
            if not self.tracks[i]:
                return
            self.tracks[i].append(e)
            # self.post_gesture(i, self.tracks[i])
            self.tracks[i] = None
            self.track_slots.discard(i)
        
        # begin guesture when touch down
        elif e.msg == HC.TOUCH_DOWN:
            if len(self.track_slots) == 2:
                return
            self.tracks[i]  = [e]
            self.track_slots.add(i)
            
        # find drag/pan/pinch/swipe guestures
        elif e.msg == HC.TOUCH_MOVE:
            if not self.tracks[i]:
                return
            self.tracks[i].append(e)

            if len(self.track_slots) == 1: # single finger move
                if e.time - self.tracks[i][0].time > 1:
                    print 'drag'
            elif len(self.track_slots) == 2: # two finger move, may be pan/pinch
                t1, t2 = [self.tracks[s] for s in self.track_slots]
                # check for pinch
                if len(t1) + len(t2) < 5:
                    return
                # make copy and check distance changing
                t1, t2 = t1[:], t2[:]
                dists = []
                while len(dists) < 5:
                    e1, e2 = t1[-1], t2[-1]
                    dx, dy = e1.x-e2.x, e1.y-e2.y
                    dists.append(dx*dx+dy*dy)
                    if e1.time < e2.time:
                        if len(t2) == 1:
                            break
                        else:
                            t2.pop()
                    else:
                        if len(t1) == 1:
                            break
                        else:
                            t1.pop()

                print dists[::-1]

    def post_gesture(self, slotid, track):
        print slotid, track

    def register(self, keycode, func):
        self.dispatch_map[keycode] = func

    def dispatch_event(self, event):
        func = self.dispatch_map.get(event.msg)
        if not func:
            return
        try:
            func(event)
        except:
            traceback.print_exc()

    def start(self):
        if self.running:
            return
        self.running = True
        t = threading.Thread(target=self.process)
        t.setDaemon(True)
        t.start()

    def stop(self):
        self.running = False

    def process(self):
        '''handle events and trigger time-related events'''
        HC = HookConstants
        timediff = 0
        while True:
            try:
                time.sleep(0.001)
                event = self.queue.get_nowait()
                self.handle_event(event)
                if timediff == 0:
                    timediff = time.time() - event.time
                if event.msg == HC.TOUCH_DOWN:
                    self.touchdowns[event.slotid] = event
                    self.touchups[event.slotid] = None
                elif event.msg == HC.TOUCH_UP:
                    self.touchups[event.slotid] = event
                    self.touchdowns[event.slotid] = None
                elif event.msg == HC.TOUCH_MOVE:
                    self.touchdowns[event.slotid] = None
                    self.touchups[event.slotid] = None
                    self.touchmoves[event.slotid] = event
            except Queue.Empty:
                if not self.running:
                    break
                now = time.time() - timediff
                for i in range(SLOT_NUM):
                    d = self.touchdowns[i]
                    u = self.touchups[i]
                    m = self.touchmoves[i]
                    # long press
                    if d is not None and now - d.time > self.long_press_delay:
                        print i, 'long press!'
                        self.handle_event(TouchPressTimeout(now, i))
                        self.touchdowns[i] = None
                    # double tap
                    if u is not None and now - u.time > self.double_tap_delay:
                        print i, 'double tap timeout!'
                        self.handle_event(TouchFollowTimeout(now, i))
                        self.touchups[i] = None
                    # move stop
                    if m is not None and now - m.time > self.move_stop_delay:
                        print i, 'move stop!'
                        self.handle_event(TouchMoveStopTimeout(now, i))
                        self.touchmoves[i] = None

            except:
                traceback.print_exc()

        print 'process done.'


class AndroidInputHookManager(object):

    def __init__(self, serial=None):
        self._serial = serial
        self._queue = Queue.Queue()
        self._listener = None
        self._parser = InputParser(self._queue)
        self._processor = GestureRecognizer(self._queue)

    def set_serial(self, serial):
        self._serial = serial

    def register(self, keycode, func):
        '''register hook function'''
        self._processor.register(keycode, func)

    def hook(self):
        '''input should be a filelike object.'''
        cmd = ['adb']
        if self._serial:
            cmd.extend(['-s', self._serial])
        cmd.extend(['shell', 'getevent', '-lt'])
        self._listener = p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        self._processor.start()

        def listen():
            while True:
                try:
                    line = p.stdout.readline().strip()
                    if not line:
                        if p.poll() is not None:
                            print 'adb terminated.'
                            self._processor.stop()
                            break
                        continue
                    self._parser.feed(line)
                except KeyboardInterrupt:
                    p.kill()
                except Exception as e:
                    p.kill()
                    print type(e), str(e)

        t = threading.Thread(target=listen)
        t.setDaemon(True)
        t.start()

    def unhook(self):
        if self._listener:
            self._listener.kill()

HookManager = AndroidInputHookManager

if __name__ == '__main__':
    hm = AndroidInputHookManager()
    hm.hook()
    while True:
        try:
            time.sleep(0.1)
        except KeyboardInterrupt:
            break
    hm.unhook()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re
import sys
import subprocess
import time
import urllib
import json
import platform
import csv

import airtest
import click
import humanize
from airtest import log2html as airlog2html
from airtest import androaxml

__debug = False

def _wget(url, filename=None):
    print 'DOWNLOAD:', url, '->', filename
    return urllib.urlretrieve(url, filename)

def _run(*args, **kwargs):
    if __debug:
        click.echo('Exec: %s [%s]' %(args, kwargs))
    kwargs['stdout'] = kwargs.get('stdout') or sys.stdout
    kwargs['stderr'] = kwargs.get('stderr') or sys.stderr
    p = subprocess.Popen(args, **kwargs)
    p.wait()

def _get_apk(config_file, cache=False):
    apk = None
    if os.path.exists(config_file):# compatiable with cli-1
        with open(config_file) as file:
            cfg = json.load(file)
            apk = cfg.get('apk')
            if not apk:
                apk = cfg.get('android', {}).get('apk_url') 
    
    if not apk:
        # sys.exit("Usage: air.test install <APK-PATH | APK-URL")
        apk = raw_input('Enter apk path or url: ')
        assert apk.lower().endswith('.apk')
        # FIXME: save to file
        with open(config_file, 'wb') as file:
            file.write(json.dumps({'apk': apk}))

    if re.match('^\w{1,2}tp://', apk):
        if cache and os.path.exists('tmp.apk'):
            return 'tmp.apk'
        _wget(apk, 'tmp.apk')
        apk = 'tmp.apk'
    return apk

@click.group()
@click.option('-v', '--verbose', is_flag=True, help='Show verbose information')
def cli(verbose=False):
    global __debug
    __debug = verbose

@cli.command(help='Check environment')
def doctor():
    # adb check
    print '>> check if contains multi adb.exe'
    paths = []
    for line in os.getenv('PATH').split(os.pathsep):
        if os.path.exists(os.path.join(line, 'adb' + '.exe' if platform.system() == 'Windows' else '')):
            paths.append(line)
    if len(paths) == 1:
        print 'Good'
    if len(paths) == 0:
        print 'No adb.exe found, download from: http://adbshell.com/download/'
    if len(paths) > 1:
        print 'adb found in %d paths, need to delete and keep one' % len(paths)
        for p in paths:
            print 'PATH:', p

@cli.command(help='Get package and activity name from apk')
@click.argument('apkfile', type=click.Path(exists=True))
def inspect(apkfile):
    pkg, act = androaxml.parse_apk(apkfile)
    click.echo('Package Name: "%s"' % pkg)
    click.echo('Activity: "%s"' % act)

@cli.command(help='Convert airtest.log to html')
@click.option('--logfile', default='log/airtest.log', help='airtest log file path',
        type=click.Path(exists=True, dir_okay=False), show_default=True)
@click.option('--listen', is_flag=True, help='open a web serverf for listen')
@click.option('--port', default=8800, help='listen port', show_default=True)
@click.argument('outdir', type=click.Path(exists=False, file_okay=False))
def log2html(logfile, outdir, listen, port):
    airlog2html.render(logfile, outdir)
    if listen:
        click.echo('Listening on port %d ...' % port)
        _run('python', '-mSimpleHTTPServer', str(port), cwd=outdir)

@cli.command(help='Take a picture of phone')
@click.option('--phoneno', help='If multi android dev connected, should specify serialno')
@click.option('--platform', default='android', type=click.Choice(['android', 'windows', 'ios']), show_default=True)
@click.option('--out', default='snapshot.png', type=click.Path(dir_okay=False),
        help='out filename [default: "snapshot.png"]', show_default=True)
def snapshot(phoneno, platform, out):
    try:
        app = airtest.connect(phoneno, device=platform)
        app.takeSnapshot(out)
    except Exception, e:
        click.echo(e)

@cli.command(help='Install apk to phone')
@click.option('--no-start', is_flag=False, help='Start app after successfully installed')
@click.option('--conf', default='air.json', type=click.Path(dir_okay=False), help='config file', show_default=True)
@click.option('-s', '--serialno', help='Specify which android device to connect')
@click.argument('apk', required=False)
def install(no_start, conf, serialno, apk):
    if not apk:
        apk = _get_apk(conf)
    adbargs = ['adb']
    if serialno:
        adbargs.extend(['-s', serialno])
    args = adbargs + ['install', '-r', apk]
    # install app
    _run(*args)

    if no_start:
        return
    pkg, act = androaxml.parse_apk(apk)
    args = adbargs + ['shell', 'am', 'start', '-n', pkg+'/'+act]
    _run(*args)

@cli.command(help='Uninstall package from device')
@click.option('--conf', default='air.json', type=click.Path(dir_okay=False), help='config file')
@click.option('-s', '--serialno', help='Specify which android device to connect')
@click.argument('apk', required=False)
def uninstall(conf, serialno, apk):
    if not apk:
        apk = _get_apk(conf, cache=True)
    pkg, act = androaxml.parse_apk(apk)
    args = ['adb']
    if serialno:
        args.extend(['-s', serialno])
    args += ['uninstall', pkg]
    _run(*args)


def main():
    cli()
    
if __name__ == '__main__':
    main()


#-*- encoding: utf-8 -*-

import cv2
import time

from atx.device.android_minicap import AndroidDeviceMinicap
from atx.record.scene_detector import SceneDetector

def test_detect():
    dev = AndroidDeviceMinicap()
    dev._adb.start_minitouch()
    time.sleep(3)

    d = SceneDetector('txxscene')
    old, new = None, None
    while True:
        # time.sleep(0.3)
        screen = dev.screenshot_cv2()
        h, w = screen.shape[:2]
        img = cv2.resize(screen, (w/2, h/2))

        # find hsv
        hsv = cv2.cvtColor(img, cv2.COLOR_BGR2HSV)
        hls = cv2.cvtColor(img, cv2.COLOR_BGR2HLS)
        _, _, V = cv2.split(hsv)
        V[V<150] = 0
        cv2.imshow('V', V)
        _, _, L = cv2.split(hls)
        L[L<150] = 0
        cv2.imshow('H', L)

        tic = time.clock()
        new = str(d.detect(img))
        t = time.clock() - tic
        if new != old:
            print 'change to', new
            print 'cost time', t
        old = new

        for _, r in d.current_scene:
            x, y, x1, y1 = r
            cv2.rectangle(img, (x,y), (x1,y1), (0,255,0) ,2)
        cv2.imshow('test', img)
        cv2.waitKey(1)

if __name__ == '__main__':
    test_detect()


#! /usr/bin/env python
# -*- coding: utf-8 -*-

import uuid

import tornado.web
import tornado.escape
from tornado import gen
from tornado.ioloop import IOLoop
from tornado.queues import Queue


# @gen.coroutine
# def consumer():
#     while True:
#         item = yield que.get()
#         try:
#             print('Doing work on %s' % item)
#             yield gen.sleep(0.5)
#         finally:
#             que.task_done()

# @gen.coroutine
# def producer():
#     for item in range(5):
#         yield que.put(item)
#         print('Put %s' % item)

# @gen.coroutine
# def main():
#     # Start consumer without waiting (since it never finishes).
#     IOLoop.current().spawn_callback(consumer)
#     yield producer()     # Wait for producer to put all tasks.
#     yield que.join()       # Wait for consumer to finish all tasks.
#     print('Done')


class MainHandler(tornado.web.RequestHandler):
    que = Queue(maxsize=2)
    results = {}

    @gen.coroutine
    def get(self, udid):
        ''' get new task '''
        item = yield self.que.get()
        print udid, item
        self.que.task_done()
        self.write(item)
        self.finish()

    @gen.coroutine
    def post(self, udid):
        ''' add new task '''
        # print self.request.body
        data = tornado.escape.json_decode(self.request.body)
        data['id'] = str(uuid.uuid1())
        print data
        yield self.que.put(data)
        self.write({'id': data['id']})
        self.finish()

    def put(self, udid):
        ''' finish task '''
        data = tornado.escape.json_decode(self.request.body)
        print data['id']
        print data['result']


def make_app(**settings):
    print settings
    return tornado.web.Application([
        (r"/rooms/([^/]*)", MainHandler),
    ], **settings)


if __name__ == '__main__':
    app = make_app(debug=True)
    app.listen(10020)
    IOLoop.current().start() #run_sync(main)


# -*- coding: utf-8 -*-
# AndroidDevice without uiautomator

import os
import re
import cv2
import time
import Queue
import struct
import socket
import warnings
import traceback
import threading
import subprocess
import collections
import numpy as np
from PIL import Image

from atx import patch
from atx.adb import Adb
from atx.device.mixin import DeviceMixin

DISPLAY_RE = re.compile(
    r'.*DisplayViewport{valid=true, .*orientation=(?P<orientation>\d+), .*deviceWidth=(?P<width>\d+), deviceHeight=(?P<height>\d+).*')
PROP_PATTERN = re.compile(
    r'\[(?P<key>.*?)\]:\s*\[(?P<value>.*)\]')

class SubAdb(Adb):
    def __init__(self, *args, **kwargs):
        super(SubAdb, self).__init__(*args, **kwargs)
        self.subs = {}
        self.touchqueue = Queue.Queue()

    def start_daemon(self, name, cmds, listener=None):
        if name in self.subs:
            p = self.subs.pop(name)
            p.kill()

        if listener is None:
            p = self.subs[name] = subprocess.Popen(cmds)
            return p

        queue = Queue.Queue()
        p = self.subs[name] = subprocess.Popen(cmds, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        
        # pull data from pipe, readline will block in subprocess
        def pull():
            while True:
                line = p.stdout.readline().strip()
                if not line:
                    if p.poll() is not None:
                        break
                    continue
                queue.put(line)
            p.stdout.close()

        t = threading.Thread(target=pull)
        t.setDaemon(True)
        t.start()

        # listen without block
        def listen():
            while True:
                try:
                    time.sleep(0.005)
                    line = queue.get_nowait()
                    listener(line)
                except Queue.Empty:
                    if p.poll() is not None:
                        break
                    continue
                except:
                    pass

        t = threading.Thread(target=listen)
        t.setDaemon(True)
        t.start()

        return p

    def start_minicap_daemon(self, params, name='minicap', port=1313, listener=None):
        if name in self.subs:
            p = self.subs.pop(name)
            p.kill()
            # print 'stop p', p.pid

        cmds = 'adb shell LD_LIBRARY_PATH=/data/local/tmp /data/local/tmp/minicap -P %s -S' % params
        p = subprocess.Popen(cmds, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        self.subs[name] = p
        # print 'start new p', p.pid
        # wait for minicap server to start
        time.sleep(3)
        subprocess.call('adb forward tcp:%s localabstract:minicap' % port)

        queue = Queue.Queue()
        # pull data from socket
        def pull():
            # print 'start pull', p.pid, p.poll()
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            s.connect(('127.0.0.1', port))
            try:
                t = s.recv(24)
                print 'minicap conntected', struct.unpack('<2B5I2B', t)
                while True:
                    frame_size = struct.unpack("<I", s.recv(4))[0]
                    trunks = []
                    recvd_size = 0
                    while recvd_size < frame_size:
                        trunk_size = min(8192, frame_size-recvd_size)
                        d = s.recv(trunk_size)
                        trunks.append(d)
                        recvd_size += len(d)
                    queue.put(''.join(trunks))
            except Exception as e:
                if not isinstance(e, struct.error):
                    traceback.print_exc()
                p.kill()
                subprocess.call('adb forward --remove tcp:%s' % port)
            finally:
                s.close()

        t = threading.Thread(target=pull)
        t.setDaemon(True)
        t.start()

        def listen():
            while True:
                try:
                    time.sleep(0.005)
                    line = queue.get_nowait()
                    listener(line)
                except Queue.Empty:
                    if p.poll() is not None:
                        break
                    continue
                except:
                    traceback.print_exc()

        t = threading.Thread(target=listen)
        t.setDaemon(True)
        t.start()

    def start_minitouch(self, port=1111):
        out = subprocess.check_output('adb shell ps -C /data/local/tmp/minitouch').strip().split('\n')
        if len(out) > 1:
            p = None
        else:
            p = self.start_daemon('minitouch', 'adb shell /data/local/tmp/minitouch')
            time.sleep(3)
            if p.poll() is not None:
                print 'start minitouch failed.'
                return
        subprocess.call('adb forward tcp:%s localabstract:minitouch' % port)

        def send():
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            s.connect(('127.0.0.1', port))
            while True:
                try:
                    time.sleep(0.005)
                    cmd = self.touchqueue.get_nowait()
                    if not cmd:
                        continue
                    elif cmd[-1] != '\n':
                        cmd += '\n'
                    s.send(cmd)
                except Queue.Empty:
                    if p is not None and p.poll() is not None:
                        print 'down'
                        break
                    continue
                except:
                    traceback.print_exc()
            s.close()
            subprocess.call('adb forward --remove tcp:%s' % port)
            if p is not None:
                p.kill()

        t = threading.Thread(target=send)
        t.setDaemon(True)
        t.start()

    def touch(self, x, y):
        print 'touch', (x, y)
        cmd = 'd 0 %d %d 30\nc\nu 0\nc\n' % (int(x), int(y))
        self.touchqueue.put(cmd)

    def swipe(self, x1, y1, x2, y2, steps=10):
        x1, y1, x2, y2 = map(int, (x1, y1, x2, y2))
        dx = (x2-x1)/steps
        dy = (y2-y1)/steps
        send = self.touchqueue.put
        send('d 0 %d %d 30\nc\n' % (x1, y1))
        for i in range(steps-1):
            x, y = x1+(i+1)*dx, y1+(i+1)*dy
            send('m 0 %d %d 30\nc\n' % (x, y))
        send('u 0 %d %d 30\nc\nu 0\nc\n' % (x2, y2))

    def pinchin(self, x1, y1, x2, y2, steps=10):
        pass

    def pinchout(self, x1, y1, x2, y2, steps=10):
        pass

    def keyevent(self, key):
        subprocess.check_call('adb shell input keyevent %s' % key)

    def home(self):
        self.keyevent('KEYCODE_HOME')

    def menu(self):
        self.keyevent('KEYCODE_MENU')

    def back(self):
        self.keyevent('KEYCODE_BACK')

    def check_output(self, *args):
        cmds = self._assemble(*args)
        output = subprocess.check_output(cmds, stderr=subprocess.STDOUT)
        return output.replace('\r\n', '\n')

    def __call__(self, *args):
        '''
        Run adb command, for example: adb(['pull', '/data/local/tmp/a.png'])

        Args:
            command: string or list of string

        Returns:
            command output
        '''
        cmds = self._assemble(*args)
        subprocess.call(cmds)

    def _assemble(self, *args):
        cmds = ['adb']
        serial = self.device_serial()
        if serial:
            cmds.extend(['-s', serial])
        cmds.extend(self.adb_host_port_options)
        cmds.extend(list(args))
        # print cmds
        return cmds

def getenv(name, default_value=None, type=str):
    value = os.getenv(name)
    return type(value) if value else default_value

class AndroidDeviceMinicap(DeviceMixin):
    def __init__(self, *args, **kwargs):
        serialno = kwargs.get('serialno', getenv('ATX_ADB_SERIALNO', None))
        self._host = kwargs.get('host', getenv('ATX_ADB_HOST', '127.0.0.1'))
        self._port = kwargs.get('port', getenv('ATX_ADB_PORT', 5037, type=int))
        self._adb = SubAdb(serialno, self._host, self._port)
        serialno = self._adb.device_serial()
        self._serial = serialno
        
        super(AndroidDeviceMinicap, self).__init__()

        self.screen_rotation = 0
        w, h = self.display
        self._screen = np.ndarray((h,w,3), dtype=np.uint8)
        self._watch_orientation()
        self.last_screenshot = None

        for action in ('keyevent', 'home', 'back', 'menu', 'touch', 'swipe', 'pinchin', 'pinchout'):
            func = getattr(self._adb, action, None)
            if func is not None:
                setattr(self, action, func)

    def _watch_screen(self):
        params = self._minicap_params()
        print 'watch screen', params
        self._adb.start_minicap_daemon(params, listener=self._update_screen)

    def _watch_orientation(self):
        out = subprocess.check_output('adb shell pm path jp.co.cyberagent.stf.rotationwatcher')
        path = out.strip().split(':')[-1]
        cmd = 'adb shell CLASSPATH="%s" app_process /system/bin "jp.co.cyberagent.stf.rotationwatcher.RotationWatcher"' % path
        self._adb.start_daemon('orientation', cmd, listener=self._update_orientation)

    def _update_orientation(self, value):
        print 'update orientation to', value
        self.screen_rotation = int(value)/90
        self._watch_screen()

    def _minicap_params(self):
        rotation = self.screen_rotation
        return '{x}x{y}@{x}x{y}/{r}'.format(
            x=self.display.width,
            y=self.display.height,
            r=rotation*90)

    def _update_screen(self, frame):
        def str2img(jpgstr):
            arr = np.fromstring(jpgstr, np.uint8)
            img = cv2.imdecode(arr, cv2.IMREAD_COLOR)
            return img
        self._screen = str2img(frame)

    def screenshot(self, filename=None):
        if filename is not None:
            cv2.imwrite(filename, self._screen)
        return Image.fromarray(self._screen[:, :, ::-1].copy())

    def screenshot_cv2(self, filename=None):
        if filename is not None:
            cv2.imwrite(filename, self._screen)
        return self._screen.copy()

    def dump_nodes(self):
        return []

    def screen2touch(self, x, y):
        '''convert touch position'''
        w, h = self.display
        ori = self.screen_rotation
        if ori == 0:
            return x, y
        elif ori == 1: # landscape-right
            return w-y, x
        elif ori == 2: # upsidedown
            return w-x, h-y
        elif ori == 3: # landscape-left
            return h-x, y
        return x, y

    @property
    def wlan_ip(self):
        """ Wlan IP """
        return self.adb_shell(['getprop', 'dhcp.wlan0.ipaddress']).strip()

    def forward(self, device_port, local_port=None):
        """Forward device port to local
        Args:
            device_port: port inside device
            local_port: port on PC, if this value is None, a port will random pick one.

        Returns:
            tuple, (host, local_port)
        """
        port = self._adb.forward(device_port, local_port)
        return (self._host, port)

    @property
    def current_package_name(self):
        return self.info['currentPackageName']

    def is_app_alive(self, package_name):
        """ Deprecated: use current_package_name instaed.
        Check if app in running in foreground """
        return self.info['currentPackageName'] == package_name

    def sleep(self, secs=None):
        """Depreciated. use delay instead."""
        self.delay(secs)

    @property
    @patch.run_once
    def display(self):
        """Virtual keyborad may get small d.info['displayHeight']
        """
        w, h = (0, 0)
        for line in self.adb_shell('dumpsys display').splitlines():
            m = DISPLAY_RE.search(line, 0)
            if not m:
                continue
            w = int(m.group('width'))
            h = int(m.group('height'))
            # o = int(m.group('orientation'))
            w, h = min(w, h), max(w, h)
            return collections.namedtuple('Display', ['width', 'height'])(w, h)

        w, h = self.info['displayWidth'], self.info['displayHeight']
        w, h = min(w, h), max(w, h)
        return collections.namedtuple('Display', ['width', 'height'])(w, h)

    @property
    def rotation(self):
        """
        Rotaion of the phone

        0: normal
        1: home key on the right
        2: home key on the top
        3: home key on the left
        """
        if self.screen_rotation in range(4):
            return self.screen_rotation
        return self.info['displayRotation']

    def adb_cmd(self, command):
        '''
        Run adb command, for example: adb(['pull', '/data/local/tmp/a.png'])

        Args:
            command: string or list of string

        Returns:
            command output
        '''
        cmds = ['adb']
        if self._serial:
            cmds.extend(['-s', self._serial])
        cmds.extend(['-H', self._host, '-P', str(self._port)])

        if isinstance(command, list) or isinstance(command, tuple):
            cmds.extend(list(command))
        else:
            cmds.append(command)
        # print cmds
        output = subprocess.check_output(cmds, stderr=subprocess.STDOUT)
        return output.replace('\r\n', '\n')

    def adb_shell(self, command):
        '''
        Run adb shell command

        Args:
            command: string or list of string

        Returns:
            command output
        '''
        if isinstance(command, list) or isinstance(command, tuple):
            return self.adb_cmd(['shell'] + list(command))
        else:
            return self.adb_cmd(['shell'] + [command])

    @property
    def properties(self):
        '''
        Android Properties, extracted from `adb shell getprop`

        Returns:
            dict of props, for
            example:

                {'ro.bluetooth.dun': 'true'}
        '''
        props = {}
        for line in self.adb_shell(['getprop']).splitlines():
            m = PROP_PATTERN.match(line)
            if m:
                props[m.group('key')] = m.group('value')
        return props

    def start_app(self, package_name):
        '''
        Start application

        Args:
            package_name: string like com.example.app1

        Returns:
            None
        '''
        self.adb_shell(['monkey', '-p', package_name, '-c', 'android.intent.category.LAUNCHER', '1'])
        return self

    def stop_app(self, package_name, clear=False):
        '''
        Stop application

        Args:
            package_name: string like com.example.app1
            clear: bool, remove user data

        Returns:
            None
        '''
        if clear:
            self.adb_shell(['pm', 'clear', package_name])
        else:
            self.adb_shell(['am', 'force-stop', package_name])
        return self

    def takeSnapshot(self, filename):
        '''
        Deprecated, use screenshot instead.
        '''
        warnings.warn("deprecated, use snapshot instead", DeprecationWarning)
        return self.screenshot(filename)

    def type(self, text):
        """Input some text, TODO(ssx): not tested.
        Args:
            text: string (text to input)
        """
        self.adb_shell(['input', 'text', text])
        return self

    def get_package_pids(self, package_name):
        pids = []
        for line in self.adb_shell(['ps | grep %s' % package_name]).splitlines():
            line = line.split()
            try:
                pids.append(int(line[1]))
            except:
                pass
        return pids



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
basic operation for a game(like a user does)
'''

import logging
logging.getLogger("paramiko").setLevel(logging.WARNING)

import subprocess
from appium import webdriver
from PIL import Image

from .. import base
from .. import patch
 
log = base.getLogger('ios')

class Monitor(object):
    def __init__(self, ip, appname):
        try:
            import paramiko
        except:
            raise RuntimeError("Require python-lib 'paramiko' installed")

        self._ssh = paramiko.client.SSHClient()
        self._ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self._ssh.connect(ip, username='root', password='alpine')
        self._ip = ip 
        self._name = appname
        def _sh(*args):
            cmd = args[0] if len(args) == 1 else subprocess.list2cmdline(args)
            stdin, out, err = self._ssh.exec_command(cmd)
            return out.read()
        self.sh = _sh

    @patch.run_once
    def ncpu(self):
        ''' number of cpu '''
        out = self.sh('sysctl', 'hw.ncpu')
        return int(out.strip().split()[1])

    @patch.run_once
    def pid(self):
        name = '/['+self._name[0]+']'+self._name[1:]  # change "grep name" -> "grep [n]ame"
        output = self.sh('ps -eo pid,command | grep '+name)
        return int(output.split()[0])

    def cpu(self):
        ''' cpu usage, range must be in [0, 100] '''
        pid = self.pid()
        output = self.sh('ps -o pcpu -p %d | tail -n+2' % pid)
        cpu = output.strip()
        return float(cpu)/self.ncpu()
        

    def memory(self):
        '''
        @description details view: http://my.oschina.net/goskyblue/blog/296798

        @param package(string): android package name
        @return dict: {'VSS', 'RSS'} (unit KB)
        '''
        output = self.sh('ps -o pmem,rss,vsz -p %d | tail -n+2' % self.pid())
        pmem, rss, vss = output.split()
        return dict(VSS=int(vss), RSS=int(rss), PMEM=float(pmem))


class Device(object):
    def __init__(self, addr=None):
        if not addr:
            addr = '127.0.0.1'
        self.url = 'http://%s:4723/wd/hub' % addr
        self.driver = webdriver.Remote(
            command_executor=self.url,
            desired_capabilities={
                'platformName': 'iOS',
                # after appium 1.2.0, deviceName is required
                'deviceName': 'ios device',
                'autoLaunch': False
            }
        )
        self._scale = None
        self.start()
        self._init()

    def _init(self):
        rw, rh = self._getShapeReal()
        w, h = self._getShapeInput()
        w, h = min(w, h), max(w, h)
        print (rw, rh), (w, h)
        self._scale = float(w)/rw
        print 'SCALE:', self._scale

    def start(self):
        self.driver.launch_app()

    def stop(self):
        self.driver.close_app()

    def clear(self):
        self.stop()

    def snapshot(self, filename):
        ''' save screen snapshot '''
        log.debug('start take snapshot')
        self.driver.save_screenshot(filename)
        log.debug('finish take snapshot and save to '+filename)
        return filename

    def _cvtXY(self, x, y):
        '''
        convert x,y from device real resolution to action input resolution
        '''
        x_input = x * self._scale #self.width / self.width_real
        y_input = y * self._scale #self.height / self.height_real
        log.debug("cvt %s,%s to %s,%s" % (x, y, x_input, y_input))
        return map(int, (x_input, y_input))

    def touch(self, x, y, duration=None):
        '''
        touch screen at (x, y)
        multi finger operation not provided yet
        '''
        if duration:
            duration *= 1000
        x, y = self._cvtXY(x, y)
        log.debug('touch position %s', (x, y))
        self.driver.tap([(x, y)], duration)

    def drag(self, (x1, y1), (x2, y2), duration=None):
        '''
        Simulate drag from (x1, y1) to (x2, y2)
        multi finger operation not provided yet
        '''
        x1, y1 = self._cvtXY(x1, y1)
        x2, y2 = self._cvtXY(x2, y2)
        log.debug('drag from (%s, %s) to (%s, %s)' % (x1, y1, x2, y2))
        if duration:
            duration = duration * 1000 # seconds to ms
        self.driver.swipe(x1, y1, x2, y2, duration)

    def _getShapeReal(self):
        '''
        Get screen real resolution
        '''
        screen_shot = self.snapshot("screen_shot.png")
        img = Image.open(screen_shot)
        self.width_real, self.height_real = img.size
        log.debug('IosDevice real resolution: width:{width}, height:{height}'.format(
            width=self.width_real, height=self.height_real))
        return img.size

    def _getShapeInput(self):
        '''
        Get screen shape for x, y Input
        '''
        screen_size = self.driver.get_window_size()
        self.width, self.height = screen_size["width"], screen_size["height"]
        log.debug('IosDevice input resolution: width:{width}, height:{height}'.format(
            width=self.width, height=self.height))
        return (self.width, self.height)

    def shape(self):
        ''' 
        Get screen width and height 
        '''
        return map(int, [p/self._scale for p in self._getShapeInput()])
        #return (self.width_real, self.height_real)

    def type(self, text):
        '''
        Input some text

        @param text: string (text want to type)
        '''
        print "not provided yet on ios"

if __name__ == '__main__':
    d = Device()
    d.snapshot("test.png")
    d.touch(180, 720)
    print d.shape()
    width, height = d.shape()
    d.drag((width * 0.5, height * 0.5), (width * 0.1, height * 0.5))
    



#-*- encoding: utf-8 -*-

import os
import time

from test_record import get_device

__dir__ = os.path.dirname(os.path.abspath(__file__))
capture_tmpdir = os.path.join(os.getcwd(), 'screenshots', time.strftime("%Y%m%d"))

def main():
    d = get_device()
    d.image_path = [os.path.join('screenshots', time.strftime("%Y%m%d"))]
    script = os.path.join(capture_tmpdir, 'steps.py')
    try:
        execfile(script)
    except Exception as e:
        raise

if __name__ == '__main__':
    main()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
 
 
#from zope.interface import Interface
from PIL import ImageGrab
import win32api,win32con,win32gui,win32process
import autopy
import ctypes,ctypes.wintypes
import pygame
import os
import time


class RECT(ctypes.Structure):# classtype for window position
    _fields_ = [('left', ctypes.c_long),
                ('top', ctypes.c_long),
                ('right', ctypes.c_long),
                ('bottom', ctypes.c_long)]
    def __str__(self):
        return str((self.left, self.top, self.right, self.bottom))

# The following two dictionaries don't contain all the characters; please check before you use them
# A dictionary to hold the characters, which may be used in text, and their corresponding key values
OriginalCodes = {
           '\n':0x0D, #enter
           ' ':0x20, #spacebar
           '\t':0x09, #TAB
           '0':0x30,
           '1':0x31,
           '2':0x32,
           '3':0x33,
           '4':0x34,
           '5':0x35,
           '6':0x36,
           '7':0x37,
           '8':0x38,
           '9':0x39,
           'a':0x41,
           'b':0x42,
           'c':0x43,
           'd':0x44,
           'e':0x45,
           'f':0x46,
           'g':0x47,
           'h':0x48,
           'i':0x49,
           'j':0x4A,
           'k':0x4B,
           'l':0x4C,
           'm':0x4D,
           'n':0x4E,
           'o':0x4F,
           'p':0x50,
           'q':0x51,
           'r':0x52,
           's':0x53,
           't':0x54,
           'u':0x55,
           'v':0x56,
           'w':0x57,
           'x':0x58,
           'y':0x59,
           'z':0x5A,
           '+':0xBB,
           ',':0xBC,
           '-':0xBD,
           '.':0xBE,
           '/':0xBF,
           '`':0xC0,
           ';':0xBA,
           '[':0xDB,
           '\\':0xDC,
           ']':0xDD,
           "'":0xDE,
           '`':0xC0}
# A dictionary which contains the characters should be typed with shift
ShiftCodes = {  
           'A':0x41,
           'B':0x42,
           'C':0x43,
           'D':0x44,
           'E':0x45,
           'F':0x46,
           'G':0x47,
           'H':0x48,
           'I':0x49,
           'J':0x4A,
           'K':0x4B,
           'L':0x4C,
           'M':0x4D,
           'N':0x4E,
           'O':0x4F,
           'P':0x50,
           'Q':0x51,
           'R':0x52,
           'S':0x53,
           'T':0x54,
           'U':0x55,
           'V':0x56,
           'W':0x57,
           'X':0x58,
           'Y':0x59,
           'Z':0x5A,
           ')':0x30,
           '!':0x31,
           '@':0x32,
           '#':0x33,
           '$':0x34,
           '%':0x35,
           '^':0x36,
           '&':0x37,
           '*':0x38,
           '(':0x39,
           '?':0xBF,
           '~':0xC0,
           ':':0xBA,
           '{':0xDB,
           '|':0xDC,
           '}':0xDD,
           "\"":0xDE}


#class Device(Interface):
class Device():
    ''' Interface documentation '''
    def __init__(self,filename=None):
        if '.exe' != filename[-4:len(filename)]:#check the name has postfix ".exe" or not; if not, add ".exe" to the end
            self.filename = filename+".exe"
        else:
            self.filename = filename
        HWND=self._getHandleThroughFilename()
        self.HWND = self._chosegamehandle(HWND)
        # print 'HWND:', self.HWND
        if not self.HWND:
            raise Exception('Can not find target application process')
        
    def _getHandleThroughFilename(self):
        Psapi = ctypes.WinDLL('Psapi.dll')
        EnumProcesses = Psapi.EnumProcesses
        EnumProcesses.restype = ctypes.wintypes.BOOL
        GetProcessImageFileName = Psapi.GetProcessImageFileNameA
        GetProcessImageFileName.restype = ctypes.wintypes.DWORD

        Kernel32 = ctypes.WinDLL('kernel32.dll')
        OpenProcess = Kernel32.OpenProcess
        OpenProcess.restype = ctypes.wintypes.HANDLE
        TerminateProcess = Kernel32.TerminateProcess
        TerminateProcess.restype = ctypes.wintypes.BOOL
        CloseHandle = Kernel32.CloseHandle
        

        MAX_PATH = 260
        PROCESS_TERMINATE = 0x0001
        PROCESS_QUERY_INFORMATION = 0x0400

        count = 32
        while True:
            ProcessIds = (ctypes.wintypes.DWORD*count)()
            cb = ctypes.sizeof(ProcessIds)
            BytesReturned = ctypes.wintypes.DWORD()
            if EnumProcesses(ctypes.byref(ProcessIds), cb, ctypes.byref(BytesReturned)):
                if BytesReturned.value<cb:
                    break
                else:
                    count *= 2
            else:
                raise Exception('Call to EnumProcesses failed')

        for index in range(BytesReturned.value / ctypes.sizeof(ctypes.wintypes.DWORD)):
            ProcessId = ProcessIds[index]
            hProcess = OpenProcess(PROCESS_TERMINATE | PROCESS_QUERY_INFORMATION, False, ProcessId)
            if hProcess:
                ImageFileName = (ctypes.c_char*MAX_PATH)()
                if GetProcessImageFileName(hProcess, ImageFileName, MAX_PATH)>0:
                    filename = os.path.basename(ImageFileName.value)
                    if filename == self.filename:
                        break
                #TerminateProcess(hProcess, 1)
                CloseHandle(hProcess)
                
        def get_hwnds_for_pid(pid):
            def callback (hwnd, hwnds):
                if win32gui.IsWindowVisible (hwnd) and win32gui.IsWindowEnabled (hwnd):
                    _, found_pid = win32process.GetWindowThreadProcessId (hwnd)
                    if found_pid == pid:
                        hwnds.append (hwnd)
                    return True
            hwnds = []
            win32gui.EnumWindows(callback, hwnds)
            return hwnds
        return get_hwnds_for_pid(ProcessId)
    
    def _chosegamehandle(self,HWND):
            if not HWND : return HWND
            else:
                for handle in HWND:
                    windowtext = win32gui.GetWindowText(handle)
                    if ":" not in windowtext: 
                        return handle
                        
    
        
    def _range(self):
        ''' Get Windows rectangle position '''
        rect = RECT()
        ctypes.windll.user32.GetWindowRect(self.HWND,ctypes.byref(rect))
        range_ = (rect.left+2,rect.top+2,rect.right-2,rect.bottom-2)
        return range_

    def _resetpt(self, x, y):
        left, top, _, _ = self._range()
        x, y = left+x, top+y
        return x, y
    
    def snapshot(self, filename=None ):
        ''' Capture device screen '''
        range_ = self._range()
        win32gui.SetForegroundWindow(self.HWND)
        time.sleep(0.1)
        pic = ImageGrab.grab(range_)
        if filename !=None:
            pic.save(filename)
        return pic
        
    def touch(self, x, y, duration=0.1):
        ''' Simulate touch '''
        (ox, oy) = self.mouseposition() # remember mouse position
        x, y = self._resetpt(x, y)
        win32api.SetCursorPos((x,y))

        win32api.mouse_event(win32con.MOUSEEVENTF_LEFTDOWN,0,0)
        time.sleep(duration)
        win32api.mouse_event(win32con.MOUSEEVENTF_LEFTUP,0,0)
        win32api.SetCursorPos((ox,oy)) # move back mouse position

        
    def drag(self, (x1, y1), (x2, y2), duration=0.5):
        ''' Simulate drag '''
        (ox, oy) = self.mouseposition() # remember mouse position
        x1, y1 = self._resetpt(x1, y1)
        x2, y2 = self._resetpt(x2, y2)
        win32api.SetCursorPos((x1, y1))
        win32api.mouse_event(win32con.MOUSEEVENTF_LEFTDOWN,0,0)
        autopy.mouse.smooth_move(x2, y2)
        win32api.mouse_event(win32con.MOUSEEVENTF_LEFTUP,0,0)
        win32api.SetCursorPos((ox,oy)) # move back mouse position
        
    def type(self, text):
        ''' Type text into device '''
        
        for c in text:
            if c in ShiftCodes:  #judge the character is a capital letter or not; 16 is the value of shift
                win32api.keybd_event(16,win32api.MapVirtualKey(16,0),0,0)
                win32api.keybd_event(ShiftCodes[c],win32api.MapVirtualKey(ShiftCodes[c],0),0,0)
                win32api.keybd_event(16,win32api.MapVirtualKey(16,0),win32con.KEYEVENTF_KEYUP,0)
                win32api.keybd_event(ShiftCodes[c],win32api.MapVirtualKey(ShiftCodes[c],0),win32con.KEYEVENTF_KEYUP,0)
            elif c in OriginalCodes:    #judge the character is a capital letter or not
                win32api.keybd_event(OriginalCodes[c],win32api.MapVirtualKey(OriginalCodes[c],0),0,0)
                win32api.keybd_event(OriginalCodes[c],win32api.MapVirtualKey(OriginalCodes[c],0),win32con.KEYEVENTF_KEYUP,0)
    
    def shape(self):
        ''' Return (width, height) '''
        pic = self.snapshot(filename=None)
        (width, height) =  pic.size
        return (width, height)
    
    def cutimage(self, filename=None):
        '''Cut picture from target window'''
        pic = self.snapshot(filename=None)
        mode = pic.mode
        size = pic.size
        data = pic.tostring()
        surface = pygame.image.fromstring(data, size, mode)
        s=pygame.display.set_mode(surface.get_size())
        box=k=0
        c=1
        while c:
            for evt in pygame.event.get():
                evt_type=evt.type
                if evt_type==5: x1,y1=evt.pos;k=1        # mouse down
                if evt_type==4 and k: x2,y2=evt.pos; box=(x1,y1,x2-x1,y2-y1) #mouse up and calculate the box range
                if evt_type==6: c=0
                s.blit(surface,(0,0))
                if box and c:pygame.draw.rect(s,0,box,1) 
                pygame.display.flip()
        q=s.subsurface(box)
        if filename!=None:
            pygame.image.save(q,filename)
        pygame.quit()
        
    def mouseposition(self):
        '''Get the current position of mouse'''
        class POINT(ctypes.Structure):
            _fields_ = [
                        ("x", ctypes.c_ulong),
                        ("y", ctypes.c_ulong)
                        ]
        point = POINT()
        ctypes.windll.User32.GetCursorPos(ctypes.byref(point))
        return point.x, point.y 
    
    def windowposition(self):
        '''Get the position of target window while given its name'''
        rect = RECT()
        ctypes.windll.user32.GetWindowRect(self.HWND,ctypes.byref(rect))
        return rect.left,rect.top,rect.right,rect.bottom
    
    def start(self, appname, extra={}):
        '''Start an app, TODO(not good now)'''
        '''appname is not used in windows interferences'''
        Path = extra.get('path')
        os.system('cd /d '+Path+' && '+'start '+self.filename)
        HWND=self._getHandleThroughFilename()
        self.HWND = self._chosegamehandle(HWND)
        if self.HWND==0:
            raise Exception(u'Target application is not successfully started')
        
    def stop(self, appname, extra={}):
        '''appname is not used in windows interferences'''
        win32gui.SendMessage(self.HWND,win32con.WM_CLOSE,0,0)
        
    def getCpu(self, appname):
        ''' Return cpu: float (Cpu usage for app) '''
        return 0.0
        
    def getMem(self, appname):
        ''' Return mem: float (unit MB, memory usage for app) '''
        return {}



#-*- encoding: utf-8 -*-

from .android_hooks import HookManager, HookConstants
from atx.record.base import BaseRecorder

class AndroidRecorder(BaseRecorder):
    def __init__(self, device=None):
        self.hm = HookManager()
        super(AndroidRecorder, self).__init__(device)
        self.hm.register(HookConstants.TOUCH_UP, self._on_click)
        # self.hm.register(HookConstants.GST_DRAG, self._on_drag)
        # self.hm.register(HookConstants.GST_SWIPE, self._on_swipe)
        # self.hm.register(HookConstants.GST_PINCH, self._on_pinch)
        # self.hm.register(HookConstants.ANY_KEY, self._on_key)

    def attach(self, device):
        if self.device is not None:
            self.detach()
        self.device = device
        self.hm.set_serial(self.device._serial)

    def detach(self):
        self.unhook()
        self.device = None
        self.hm.set_serial(None)

    def hook(self):
        self.hm.hook()

    def unhook(self):
        self.hm.unhook()

    def _on_click(self, event):
        pos = (event.x, event.y)
        print 'touch on', pos
        self.on_click(pos)


from airtest import jsonlog
import json
import os

def test_logdict():
    log = jsonlog.JSONLog('test.log')
    log.writeline({'type':'message'})
    j = json.loads(open('test.log').readlines()[-1])
    assert j.get('type') == 'message'
    os.unlink('test.log')




# coding: utf-8

from __future__ import absolute_import

import collections

from atx import imutils
from atx import strutils


FindPoint = collections.namedtuple('FindPoint', ['pos', 'confidence', 'method', 'matched'])
Display = collections.namedtuple('Display', ['width', 'height'])


__boundstuple = collections.namedtuple('Bounds', ['left', 'top', 'right', 'bottom'])
class Bounds(__boundstuple):
    def __init__(self, *args, **kwargs):
        super(Bounds, self).__init__(*args, **kwargs)
        self._area = None

    def is_inside(self, x, y):
        v = self
        return x > v.left and x < v.right and y > v.top and y < v.bottom

    @property
    def area(self):
        if not self._area:
            v = self
            self._area = (v.right-v.left) * (v.bottom-v.top)
        return self._area

    @property
    def center(self):
        v = self
        return (v.left+v.right)/2, (v.top+v.bottom)/2

    def __mul__(self, mul):
        return Bounds(*(int(v*mul) for v in self))


class Pattern(object):
    def __init__(self, image, offset=(0, 0), anchor=0, rsl=None, resolution=None):
        """
        Args:
            image: image filename or image URL
            offset: offset of image center
            anchor: not supported
            resolution: image origin screen resolution
            rsl: alias of resolution
        """
        self._name = image if isinstance(image, basestring) else 'unknown'
        self._image = imutils.open(image)
        self._offset = offset
        self._resolution = rsl or resolution
        if isinstance(image, basestring):
            self._name = image

    def __str__(self):
        return 'Pattern(name: {}, offset: {})'.format(strutils.encode(self._name), self.offset)
    
    @property
    def image(self):
        return self._image

    @property
    def offset(self):
        return self._offset

    @property
    def resolution(self):
        return self._resolution


#-*- encoding: utf-8 -*-
# I'm Shakespeare!

import re
import cv2
import time
import warnings
import traceback
from random import randint, choice

from scene_detector import SceneDetector

class Reporter(object):

    def prepare(self, device, package=None, pids=None):
        '''called before loop. initialize device related stuff.'''
        raise NotImplementedError()

    def collect(self):
        '''called every run. collect logs.'''
        raise NotImplementedError()

    def dump(self):
        '''called after loop. dump logs.'''
        raise NotImplementedError()

class AdbLineReporter(Reporter):

    name = 'shell'
    filter_by = None

    def __init__(self):
        self.buffer = []
        # cache grep condition
        self.package = None
        self.pids = None
        self.grep = None
        self.prepared = False

    def prepare(self, device, package=None, pids=None):
        self.device = device
        self.package = package
        self.pids = pids
        if self.filter_by == 'package' and package is not None:
            self.grep = re.compile('%s' % package) # there may be dot in package name but that's ok. 
        elif self.filter_by == 'pids' and pids is not None:
            self.grep = re.compile('|'.join([str(p) for p in pids]))
        self.prepared = True

    def collect(self):
        if not self.prepared:
            return
        cmd = self.command()
        lines = self.device.adb_shell(cmd).split('\n')
        # print cmd, len(lines), lines[0]
        for line in lines:
            line = line.strip()
            if not line:
                continue
            if self.grep is not None and self.grep.search(line) is None:
                continue
            self.process_line(line)
            self.buffer.append(line)

    def process_line(self, line):
        pass

    def dump(self):
        if not self.buffer:
            print 'nothing to dump'
            return
        filename =  '%s_%s.log' % (self.name, time.strftime('%m%d_%H%M%S'))
        if self.package is not None:
            filename = '%s_%s' % (self.package, filename)
        print 'dump report to', filename
        with open(filename, 'w') as f:
            f.write('\n'.join(self.buffer))

class LogcatReporter(AdbLineReporter):

    name = 'logcat'
    timefmt = '%m-%d %H:%M:%S.0'
    timepat = re.compile('\d\d-\d\d\s+\d\d:\d\d:\d\d\.\d+')
    filter_by = 'pids'

    def __init__(self):
        super(LogcatReporter, self).__init__()
        self.timestr = time.strftime(self.timefmt, time.localtime())

    def prepare(self, device, package=None, pids=None):
        super(LogcatReporter, self).prepare(device, package, pids)
        self.device.adb_shell('logcat -c')

    def command(self):
        return "logcat -t '%s' -v time" % self.timestr

    def process_line(self, line):
        m = self.timepat.search(line)
        if m:
            self.timestr = m.group()
        # TODO
        # the last digits should be increased by 1,
        # or there will be some duplicated lines.

# TODO: clean anr/traces.txt on non-root devices.
class AnrTraceReporter(AdbLineReporter):
    name = 'anr'

    def command(self):
        return 'cat /data/anr/traces.txt'

_default_reporters = (LogcatReporter,)

class Monkey(object):

    actions = ('touch', 'swipe', 'pinchin', 'pinchout', 'home', 'menu', 'back')
    delay = 0.5

    def __init__(self, probs):
        total = sum(probs.values())
        self.weights = []
        accum = 0
        for i in range(len(self.actions)):
            a = self.actions[i]
            w = probs.pop(a, 0)
            self.weights.append(int(accum*10000./total))
            accum += w
        self.weights.append(int(accum*10000./total))
        if probs:
            warnings.warn('Unsupported actions: %s' % probs.keys())

        self.device = None
        self.reporters = [r() for r in _default_reporters]

    def run(self, device, package=None, maxruns=None):
        self.device = device
        pids = None
        if package is not None:
            self.device.start_app(package)
            time.sleep(1)
            pids = self.device.get_package_pids(package)

        for reporter in self.reporters:
            reporter.prepare(device, package, pids)

        count = 0
        while maxruns is None or count < maxruns:
            try:
                time.sleep(self.delay)
                self.next_action()
                for reporter in self.reporters:
                    reporter.collect()
            except KeyboardInterrupt:
                break
            except:
                traceback.print_exc()
            count += 1

        for reporter in self.reporters:
            reporter.dump()

    def next_action(self):
        r = randint(1, 10000)
        for i in range(len(self.actions)-1):
            if r <= self.weights[i+1]:
                break
        a = self.actions[i]

        func = getattr(self, 'do_%s' % a, None)
        if func is not None:
            func()
        else:
            print 'unknown action', a

    def do_touch(self):
        w, h = self.device.display
        x, y = randint(1, w), randint(1, h)
        self.device.touch(x, y)

    def do_swipe(self):
        w, h = self.device.display
        x1, y1, x2, y2 = randint(1, w), randint(1, h), randint(1, w), randint(1, h)
        self.device.swipe(x1, y1, x2, y2)

    # def do_pinchin(self):
    #     w, h = self.device.display
    #     angle = randint(0, 360)

    # def do_pinchout(self):
    #     w, h = self.device.display
    #     angle = randint(0, 360)

    def do_home(self):
        self.device.home()

    def do_menu(self):
        self.device.menu()

    def do_back(self):
        self.device.back()

class StupidMonkey(Monkey):
    '''find touchables through hard work'''

    movestep = 30 #pixels

    def __init__(self, probs, scene_directory):
        super(StupidMonkey, self).__init__(probs)
        self.scene_detector = SceneDetector(scene_directory)
        self.scene_touches = {}
        self.scene_rects = {}

    def get_current_scene(self):
        screen = self.device.screenshot_cv2()
        h, w = screen.shape[:2]
        img = cv2.resize(screen, (w/2, h/2))
        scene = self.scene_detector.detect(img)

        for rect in self.scene_rects.get(str(scene),{}).itervalues():
            l, t, r, b = rect
            cv2.rectangle(img, (l, t), (r, b), 255, 2)
        cv2.imshow('scene', img)
        cv2.waitKey(1)
        return scene

    def do_touch(self):
        # width, height = self.device.display
        width, height = 1080, 1920
        scene = self.get_current_scene()
        if scene is None:
            # fall back to random point
            x, y = randint(1, width), randint(1, height)
        else:
            pos = self.scene_touches.get(str(scene), 0)
            w, h = width/self.movestep, height/self.movestep # grid points
            dy, dx = divmod(pos, w-1)
            if dy >= h-1:
                # TODO: return a random clickable point
                x, y = randint(1, width), randint(1, height)
            else:
                x, y = (dx+1)*self.movestep, (dy+1)*self.movestep
                self.scene_touches[str(scene)] = pos+1

        self.last_touch_point = x, y
        self.device.touch(x, y)

        # watch what happend after touch
        if scene is None:
            return
        newscene = self.get_current_scene()
        if newscene is None or newscene == scene:
            return
        s1 = str(scene)
        s2 = str(newscene)
        if s1 not in self.scene_rects:
            self.scene_rects[s1] = {}
        if s2 not in self.scene_rects[s1]:
            self.scene_rects[s1][s2] = (x, y, x, y)
        else:
            l, t, r, b = self.scene_rects[s1][s2]
            l, r = min(l, x), max(r, x)
            t, b = min(t, y), max(t, y)
            self.scene_rects[s1][s2] = (l, t, r, b)

    def do_swipe(self):
        pass

class RandomContourMonkey(Monkey):

    def do_touch(self):
        width, height = 1080, 1920

        screen = self.device.screenshot_cv2()
        h, w = screen.shape[:2]
        img = cv2.resize(screen, (w/2, h/2))
        gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)

        edges = cv2.Canny(gray, 80, 200)
        _, thresh = cv2.threshold(edges, 0, 255, cv2.THRESH_OTSU)
        contours, _ = cv2.findContours(thresh, cv2.RETR_CCOMP, cv2.CHAIN_APPROX_SIMPLE)
        contours.sort(key=lambda cnt: len(cnt), reverse=True)

        rects = []
        for cnt in contours:
            hull = cv2.convexHull(cnt)
            hull_area = cv2.contourArea(hull)
            x,y,w,h = cv2.boundingRect(cnt)
            rect_area = float(w*h)
            if w<20 or h<20 or rect_area<100:
                continue
            if hull_area/rect_area < 0.50:
                continue
            rects.append((x, y, x+w, y+h))
            cv2.rectangle(img, (x, y), (x+w, y+h), 255, 2)

        if not rects:
            x, y = randint(1, width), randint(1, height)
        else:
            x1, y1, x2, y2 = choice(rects)
            x, y = randint(x1, x2), randint(y1, y2)
            cv2.rectangle(img, (x1, y1), (x2, y2), (0, 255, 0), 2)

        x, y = self.device.screen2touch(x*2, y*2)
        self.device.touch(x, y)
        cv2.imshow('img', img)
        cv2.waitKey(1)

    def do_swipe(self):
        pass

if __name__ == '__main__':
    pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import atexit
import os
import time
import json

from atx import consts


__dir__ = os.path.dirname(os.path.abspath(__file__))

def listen(d, save_dir='report'):
    image_dir = os.path.join(save_dir, 'images')
    if not os.path.exists(image_dir):
        os.makedirs(image_dir)

    steps = []
    w, h = d.display
    if d.rotation in (1, 3): # for horizontal
        w, h = h, w
    result = dict(device=dict(
        display=dict(width=w, height=h),
        serial=d.serial,
    ), steps=steps)
    start_time = time.time()

    def listener(evt):
        if evt.flag == consts.EVENT_CLICK:
            screen_before = 'images/before_%d.png' % time.time()
            screen_before_abspath = os.path.join(save_dir, screen_before)
            d.last_screenshot.save(screen_before_abspath)
            screen_after = 'images/after_%d.png' % time.time()
            d.screenshot(os.path.join(save_dir, screen_after))

            (x, y) = evt.args
            steps.append({
                'time': '%.1f' % (time.time()-start_time,),
                'action': 'click',
                'screen_before': screen_before,
                'screen_after': screen_after,
                'position': {'x': x, 'y': y},
                'success': True,
            })

        # print 'EVENT:', evt

    d.add_listener(listener, consts.EVENT_ALL ^ consts.EVENT_SCREENSHOT)

    def on_finish():
        data = json.dumps(result)
        tmpl_path = os.path.join(__dir__, 'index.tmpl.html')
        save_path = os.path.join(save_dir, 'index.html')

        with open(tmpl_path) as f:
            html_content = f.read().replace('$$data$$', data)

        with open(save_path, 'wb') as f:
            f.write(html_content)

    atexit.register(on_finish)


# -*- coding: utf-8 -*-
# hook only ui related events via uiautomator.
# the basic idea is find gesture target via postion.
# so we should simulate the layout via dumped ui tree.
# The ui hierarchy needs to be dumped repeatedly 
# and the process costs a rather long time.
# after each gesture we need to refresh the tree.


import re
import cv2
import subprocess
import collections
import xml.dom.minidom
import numpy as np

from atx.device import Bounds
UINode = collections.namedtuple('UINode', [
    'xml', 'children', 'depth',
    'bounds', 
    'selected', 'checkable', 'clickable', 'scrollable', 'focusable', 'enabled', 'focused', 'long_clickable',
    'password',
    'class_name',
    'index', 'resource_id',
    'text', 'content_desc',
    'package'])

def parse_bounds(text):
    m = re.match(r'\[(\d+),(\d+)\]\[(\d+),(\d+)\]', text)
    if m is None:
        return None
    return Bounds(*map(int, m.groups()))

def str2bool(v):
    return v.lower() in ("yes", "true", "t", "1")

def convstr(v):
    return v.encode('utf-8')

class AndroidLayout(object):
    def __init__(self):
        self.tree = None
        self.nodes = []
        self.rotation = 0

    def find_clickable_rect(self, x, y):
        def _find(node, x, y):
            if node.enabled and node.bounds.is_inside(x, y):
                for n in node.children[::-1]:
                    subnode = _find(n, x, y)
                    if subnode and subnode.clickable:
                        return subnode
                return node
        return _find(self.tree, x, y)

    def find_minimal_clickable_rect(self, x, y):
        for n in self.nodes:
            if n.enabled and n.clickable and n.bounds.is_inside(x, y):
                return n

    def find_scrollable_rect(self, x, y):
        for n in self.nodes:
            if n.enabled and n.scrollable and n.bounds.is_inside(x, y):
                return n

    def display(self):
        self.dump_nodes()
        if not self.tree or not self.nodes:
            return
        b = self.tree.bounds
        w, h = b.right - b.left, b.bottom - b.top
        img = np.zeros((h, w, 3), np.uint8)
        highlight = np.zeros((h, w, 3), np.uint8)
        
        i = 0
        for n in self.nodes:
            if not n.clickable and not n.scrollable: continue
            b = n.bounds
            cv2.rectangle(img, (b.left, b.top), (b.right, b.bottom), (83, min(255, i*5), 18), 2)
            i += 1

        def on_mouse(event, x, y, flags, param):
            highlight, down, move = param
            if event == cv2.EVENT_LBUTTONDOWN:
                param[1] = (x, y)
                param[2] = False
                return
            if event == cv2.EVENT_MOUSEMOVE:
                if param[2]: return
                if param[1] is None:
                    param[2] = False
                    return
                _x, _y = param[1]
                if (_x-x)**2 + (_y-y)**2 > 64:
                    param[2] = True
                return
            if event != cv2.EVENT_LBUTTONUP: return
            param[0][:,:,:] = 0
            if param[1] and param[2]: # drag
                node = self.find_scrollable_rect(x*2, y*2)
                if node:
                    b = node.bounds
                    print 'scroll to', x*2, y*2, b, node.depth
                    cv2.rectangle(param[0], (b.left, b.top), (b.right, b.bottom), (0,255,255), 4)
            else:
                # node = self.find_clickable_rect(x*2, y*2)
                node = self.find_minimal_clickable_rect(x*2, y*2)
                if node:
                    b = node.bounds
                    print 'click at', x*2, y*2, b, node.depth
                    cv2.rectangle(param[0], (b.left, b.top), (b.right, b.bottom), (0,0,255), 4)
            param[1], param[2] = None, False

        cv2.namedWindow("layout")
        cv2.setMouseCallback('layout', on_mouse, [highlight, None, False])

        while True:
            try:
                disp = img + highlight
                disp = cv2.resize(disp, (w/2, h/2))
                cv2.imshow('layout', disp)
                cv2.waitKey(1)
            except KeyboardInterrupt:
                break

    def _parse_xml_node(self, node, depth=0):
        # ['bounds', 'checkable', 'class', 'text', 'resource_id', 'package']
        __alias = {
            'class': 'class_name',
            'resource-id': 'resource_id',
            'content-desc': 'content_desc',
            'long-clickable': 'long_clickable',
        }

        parsers = {
            'bounds': parse_bounds,
            'text': convstr,
            'class_name': convstr,
            'resource_id': convstr,
            'package': convstr,
            'checkable': str2bool,
            'scrollable': str2bool,
            'focused': str2bool,
            'clickable': str2bool,
            'enabled': str2bool,
            'selected': str2bool,
            'long_clickable': str2bool,
            'focusable': str2bool,
            'password': str2bool,
            'index': int,
            'content_desc': convstr,
        }
        ks = {}
        for key, value in node.attributes.items():
            key = __alias.get(key, key)
            f = parsers.get(key)
            if value is None:
                ks[key] = None
            elif f:
                ks[key] = f(value)
        for key in parsers.keys():
            ks[key] = ks.get(key)
        ks['children'] = []
        ks['depth'] = depth
        ks['xml'] = node

        return UINode(**ks)

    def dump_nodes(self):
        subprocess.check_call('adb shell uiautomator dump /data/local/tmp/window_dump.xml')
        subprocess.check_call('adb pull /data/local/tmp/window_dump.xml')
        xmldata = open('window_dump.xml').read()
        dom = xml.dom.minidom.parseString(xmldata)
        root = dom.documentElement
        self.rotation = int(root.getAttribute('rotation'))

        def walk(node, ui_nodes, depth=0):
            while len(node.childNodes) == 1 and node.getAttribute('bounds') == '':
                node = node.childNodes[0]
                depth += 1
            uinode = self._parse_xml_node(node, depth)
            for n in node.childNodes:
                sub = walk(n, ui_nodes, depth+1)
                if sub is not None:
                    uinode.children.append(sub)
            ui_nodes.append(uinode)
            return uinode

        self.nodes = []
        self.tree = walk(root, self.nodes)
        # self.nodes.sort(key=lambda x: x.depth, reverse=True)
        self.nodes.sort(key=lambda x: x.bounds.area)

if __name__ == '__main__':
    layout = AndroidLayout()
    nodes = layout.dump_nodes()
    layout.display()  
